
#include "anynucleus.h"
#include "particle.h"
#include <algorithm>  
   
using namespace std;


static inline double bessj0(double x)
{
 if(fabs(x)<0.000001) 
   return cos(x);
 else
   return sin(x)/x;
}


///////////////////////////////////////////////////////////////////////////////
/// Construct nucleus from params given in the input file 
///////////////////////////////////////////////////////////////////////////////
anynucleus::anynucleus(params &par):
		    nucleus(par),
		    Eb(par.nucleus_E_b*MeV),
		    //local_kf(par.nucleus_local_kf*MeV),
		    kf(par.nucleus_kf*MeV)
		    //bodek(par.nucleus_bodek)
{ 
  using namespace PDG;
  if(p+n==1)
    Eb=kf=local_kf=0;
  pr=p;
  nr=n;	 
  double A = n+p;			
  double Ap = pow(A, 1.0 / 3);
  _V = sqrt(mass_proton*mass_proton + kf*kf) - mass_proton + 7*MeV;
  //potencjal, wartosc do ustalenia
  _r=calculate_radius();
}

///////////////////////////////////////////////////////////////////////////////
/// calculate radius  
///////////////////////////////////////////////////////////////////////////////
double anynucleus::calculate_radius()
{ double r0=0, r1=12*fermi,rs=0;
  double d0=0.01*density(0,r0);
  double fermi3=fermi*fermi*fermi;
  double oldrs;
  do
  { oldrs=rs;
    rs=(r0+r1)/2;
    double ds=density(0,rs);
    if(ds>d0)
      r0=rs;
    else
      r1=rs;
//    cout<<r0/fermi<<" "<< rs/fermi<< " " <<r1/fermi<<" "<< ds*fermi3<<" "<<d0*fermi3<<endl;
  }
  while(rs!=oldrs); 

 return rs; 
}



static double MI2(double rms2, double q,double r)
{ double qr=q*r;
double r2=r*r;
double cosqr=cos(qr);
double sinqr=sin(qr);
return (((((cosqr*qr/6-sinqr/2)*qr-cosqr)*qr+sinqr)*rms2/r2 + sinqr)/r - cosqr*q)/r2;
}

static double MI(double rms2, double q,double r)
{ double qr=q*r;
double r2=r*r;
double r3=r2*r;
double cosqr=cos(qr);
double sinqr=sin(qr);
return (-cosqr*qr+sinqr)/r3
    //   + (((cosqr*qr/6-sinqr/2)*qr -cosqr)*qr+sinqr)*rms2/r2/r3;
    +(cosqr*qr*qr-2*cosqr-2*sinqr*qr)/r2/r2/6; 
}


static double prfMI(double rms,double q1,double q2, double r)
{ static double const twoPi2=2*Pi*Pi;
if(r==0) r=0.00001; 
rms*=rms; 
return max(0.0, (MI(rms,q2,r)-MI(rms,q1,r))/twoPi2);
}


///density profile for the HO model 
static double prfH0(double a,double alfa,double rho0, double r)
{double x=r/a,x2=x*x; 
return rho0*(1+alfa*x2)*exp(-x2);
}

///density profile for the MHO model
static double prfMH0(double a,double alfa,double rho0, double r)
{double x=r/a,x2=x*x; 
return (rho0*(1+alfa*x2+alfa)*exp(-x2));
}

///density profile for the 2pF model
static double prf2pf(double a,double alfa,double rho0, double r)
{//double x=r/a,x2=x*x;
return rho0/(1+exp((r-a)/alfa));
}

///density profile for the 3pF model
static double prf3pf(double a,double alfa, double w, double rho0, double r)
{double x=r/a,x2=x*x;
return max(0.,rho0*(1+w*x2)/(1+exp(r-a)/alfa)); 
}

///density profile for the 3pG model
static double prf3pg(double a,double alfa, double w, double rho0, double r)
{double x=r/a, x2=x*x;
return rho0*(1+w*x2)/(1+exp((r*r-a*a)/(alfa*alfa)));
}

///density profile for the FB model
static double prfFB(double fb[],  double r)
{ double R=fb[0];
if(r>R)
    return 0;
  double suma=0; 
  double x=Pi*r/R;
  for(int j=1; j<=17 and fb[j] ; j++)
    {   suma+=fb[j]*bessj0(j*x);
    }	
  return max(0.,suma);	
}


///density profile for the SOG model
double anynucleus::prfSOG(double sog[],double r)
{ static const double twoPi32=2*Pi*sqrt(Pi);
  double rms=sog[0];
  double g=sog[1]/sqrt(1.5);
  double coef=twoPi32*g*g*g;
  double suma=0;
  for(int j=2; j<=26 ; j+=2)
    {  double Ri=sog[j];
       double Qi=sog[j+1];
       double Ai=Qi/(coef*(1+2*pow2(Ri/g)));

       suma+=Ai*(exp(-pow2((r-Ri)/g))+exp(-pow2((r+Ri)/g)));
    }	
  return max(0.,suma*(nr+pr));	
}


///////////////////////////////////////////////////////////////////////////////
/// proton density at dist r from the center
///////////////////////////////////////////////////////////////////////////////
double anynucleus::proton_density(double r)
{ return density(pdg_proton,r);
}

///////////////////////////////////////////////////////////////////////////////
/// neutron density at dist r from the center
///////////////////////////////////////////////////////////////////////////////
double anynucleus::neutron_density(double r)
{ return density(pdg_neutron,r);
}

///////////////////////////////////////////////////////////////////////////////
/// calculates nuclear density of proton. neutrons at radius r from the center
///////////////////////////////////////////////////////////////////////////////
double anynucleus::density(int pdg, double r)
{ r/=fermi;
// He3
static double fb201[18]={5, 0.20020e-1, 0.41934e-1, 0.36254e-1, 0.17941e-1, 0.46608e-2,
                            0.46834e-2, 0.52042e-2, 0.38280e-2, 0.25661e-2, 0.14182e-2,
                            0.61390e-3, 0.22929e-3};

//N15
static double fb708[18]={7, 0.25491e-1, 0.50618e-1, 0.29822e-1, -0.55196e-2, -0.15913e-1, 
                           -0.76184e-2,-0.23992e-2,-0.47940e-3, 0, 0, 0, 0, 0, 0, 0, 0, 
			  0};

//C12
static double fb606[18]={8, 0.15721e-1, 0.38732e-1, 0.36808e-1, 0.14671e-1,-0.43277e-2, 
                           -0.97752e-2,-0.68908e-2,-0.27631e-2,-0.63568e-3, 0.71809e-5,
                            0.18441e-3, 0.75066e-4, 0.51069e-4, 0.14308e-4, 0.23170e-5,
                            0.68465e-6, 0};
//
static double fb1618[18]={8,0.37036e-1, 0.58506e-1, 0.12082e-1,-0.19022e-1,-0.83421e-2,
                            0.45434e-2, 0.28346e-2,-0.52304e-3, 0.27754e-4, 0.59403e-4,
                           -0.42794e-4, 0.20407e-4,-0.79934e-5, 0.27354e-5,-0.83914e-6,
                            0, 0};

static double fb1620[18]={8,0.37032e-1, 0.57939e-1, 0.10049e-1,-0.19852e-1,-0.67176e-2,
                            0.61882e-2, 0.37795e-2,-0.55272e-3,-0.12904e-3, 0.15845e-3,
                           -0.84063e-4, 0.34010e-4,-0.11663e-4, 0.35204e-5, 0.95135e-6,
                            0, 0};

static double fb2228[18]={9.5, 0.31818e-1, 0.58556e-1, 0.19637e-1,-0.24309e-1,-0.18748e-1,
                               0.33741e-2, 0.89961e-2, 0.37954e-2,-0.41238e-3, 0.12540e-3,
                               0, 0, 0, 0, 0, 
                               0, 0};
//Cr54
static double fb2430[18]={9.0, 0.39002e-1, 0.60305e-1, 0.45845e-2,-0.30723e-1,-0.91355e-2,
			       0.93251e-2, 0.60583e-2,-0.15602e-2,-0.76809e-3, 0.76809e-3, 
			       -0.34804e-3, 0, 0,0,0,0,0};


static double fb3242[18]={10,  0.37989e-1, 0.58298e-1, 0.27406e-2,-0.30666e-1,-0.81505e-2,
                               0.10231e-1, 0.49382e-2,-0.16270e-2,-0.13937e-2, 0.15476e-3,
                               0.14396e-3,-0.73075e-4, 0.31998e-4,-0.12822e-4, 0.48406e-5,
                               0, 0};

static double fb3244[18]={10,  0.37951e-1, 0.57876e-1, 0.15303e-2,-0.31822e-1,-0.76875e-2,
                               0.11237e-1, 0.50780e-2,-0.17293e-2,-0.15523e-2, 0.72439e-4,
                               0.16560e-3,-0.86631e-4, 0.39159e-4,-0.16259e-4, 0.63681e-5,
                               0, 0};

static double fb4252[18]={12,  0.30661e-1, 0.58828e-1, 0.20396e-1,-0.28830e-1,-0.25077e-1, 
                               0.44768e-2, 0.13127e-1, 0.19548e-2,-0.61403e-2,-0.35825e-2, 
                               0.73790e-3, 0.61882e-3,-0.40556e-3,-0.55748e-5,-0.12453e-3, 
                              -0.57812e-4,-0.21657e-4};

static double fb4254[18]={12,  0.30564e-1, 0.58013e-1, 0.19255e-1,-0.28372e-1,-0.23304e-1, 
                               0.49894e-2, 0.12126e-1, 0.10496e-2,-0.62592e-2,-0.32814e-2, 
                               0.89668e-3, 0.50636e-3,-0.43412e-3, 0.71531e-4, 0.76745e-4,
                              -0.54316e-4, 0.23386e-6};

static double fb4256[18]={12,  0.30483e-1, 0.57207e-1, 0.17888e-1,-0.28388e-1,-0.21778e-1, 
                               0.56780e-2, 0.11236e-1, 0.82176e-3,-0.50390e-2,-0.23877e-2, 
                               0.71492e-3, 0.29839e-3,-0.31408e-3, 0.80177e-3, 0.43682e-4,
                              -0.51394e-4, 0.22293e-4};
//Mo100
static double fb4258[18]={12,  0.30353e-1, 0.56087e-1, 0.16057e-1,-0.28767e-1,-0.20683e-1,
			       0.62429e-2, 0.11058e-1, 0.11502e-2,-0.39395e-2,-0.14978e-2, 
			       0.76350e-3, 0.10554e-3,-0.25658e-3, 0.10964e-3, 0.10015e-4,
			      -0.40341e-4, 0.25744e-4};
//Pd104
static double fb4658[18]={11,  0.41210e-1, 0.62846e-1,-0.21202e-2,-0.38359e-1,-0.44693e-2,
			       0.16656e-1, 0.36873e-2,-0.57534e-2,-0.32499e-2, 0.69844e-3,
			       0.16304e-2, 0.59882e-3, 0, 0, 0,
			       0, 0};
//Pd106
static double fb4660[18]={11,  0.41056e-1, 0.61757e-1,-0.29891e-2,-0.37356e-1,-0.35348e-2,
			       0.16085e-1, 0.28502e-2,-0.55764e-2,-0.15433e-2, 0.22281e-2,
			       0.13160e-2, 0.16508e-4, 0, 0, 0,
			       0,0};
//Pd108
static double fb4662[18]={11,  0.40754e-1, 0.59460e-1,-0.54077e-2,-0.36305e-1,-0.21987e-2,
			       0.15418e-1, 0.25927e-2,-0.52781e-2,-0.19757e-2, 0.10339e-2,
			       0.22891e-3,-0.33464e-3,0, 0, 0,
			       0, 0};

// Sum Of Gausians  { rms, RP, R1, Q1,R2, Q2, ...,R12 ,Q12}
// H3
static double sog102[26]={1.764,0.8,
                          0.0,0.035952,
                          0.2,0.027778,
                          0.5,0.131291,
                          0.8,0.221551,
                          1.2,0.253691,
                          1.6,0.072905,
                          2.0,0.152243,
                          2.5,0.051564,
                          3.0,0.053023,
                          0,0,
                          0,0,
                          0,0};


static double sog202[26]={1.6768,1.00,
                          0.2, 0.034724,
                          0.6, 0.430761,
                          0.9, 0.203166,
                          1.4, 0.192986,
                          1.9, 0.083866,
                          
                          2.3, 0.033007,
                          2.6, 0.014201,
                          3.1, 0.000000,
                          3.5, 0.006860,
                          4.2, 0.000000,
                          
                          4.9, 0.000438,
                          5.2, 0.000000
                          };

  double A=p+n; // the size of the nucleus will not change (only the density)
  double s; 

  switch(pdg)
  {case 0:          s=double(pr+nr)/A;    break;
  case pdg_proton:  s=double(pr)/A; break;
  case pdg_neutron: s=double(nr)/A; break;
  default:          return 0;
  }
  s/=(fermi*fermi*fermi);

 double r2=r*r; 
 double r5=r2*r2*r;
 
 double m=1;
 int kod=p*100+n;
 

while(true) 
{int a=kod/100;
//cout<<"DENSITY "<<kod<<endl;
switch(kod)
{
case 0: return 0;
case 1: return 0;
case 100: return 0;
case 304: return m*s*prfH0(1.772, 0.327, 0.151583,r);
case 405: return m*s*prfH0(1.776, 0.631, 0.148229,r);
case 505: return m*s*prfH0(1.718, 0.837, 0.157023,r);
case 506: return m*s*prfH0(1.698, 0.811, 0.182048,r);
case 808: return m*s*prfH0(1.83314, 1.544, 0.140667,r);
case 809: return m*s*prfH0(1.79818, 1.498, 0.161712,r);
case 810: return m*s*prfH0(1.84114, 1.513, 0.158419,r);

//Modified harmonic-oscillator model (MHO):liczba protonow, liczba neutronow, a, alfa, rho0
case 607: return m*s*prfMH0(1.6359, 1.403, 0.118308, r);
case 608: return m*s*prfMH0(1.734, 1.3812, 0.108294, r);

//Two - parameter Fermi model (2pF):liczba protonow, liczba neutronow, c=a, z=alfa, rho0
case  910: return m*s*prf2pf(2.58,   0.567,  0.178781,r);
case 1010: return m*s*prf2pf(2.740,  0.572,  0.162248,r);
case 1012: return m*s*prf2pf(2.782,  0.549,  0.176167,r);
case 1214: return m*s*prf2pf(3.05,   0.523,  0.16955, r);
case 1314: return m*s*prf2pf(2.84,   0.569,  0.201502,r);
case 1818: return m*s*prf2pf(3.54,   0.507,  0.161114,r);
case 1822: return m*s*prf2pf(3.53,   0.542,  0.176112,r);
case 2226: return m*s*prf2pf(3.84315,0.5885, 0.163935,r);
case 2328: return m*s*prf2pf(3.94,   0.505,  0.17129,r);
case 2426: return m*s*prf2pf(3.941,  0.5665, 0.161978,r);
case 2428: return m*s*prf2pf(4.01015,0.5785, 0.159698,r);
case 2429: return m*s*prf2pf(4.000,  0.557,  0.165941,r);
case 2530: return m*s*prf2pf(3.8912, 0.567,  0.184036,r);//dopisane
case 3025: return m*s*prf2pf(3.8912, 0.567,  0.184243,r);
case 2628: return m*s*prf2pf(4.074,  0.536,  0.162833,r);
case 2630: return m*s*prf2pf(4.111,  0.558,  0.162816,r);
case 2632: return m*s*prf2pf(4.027,  0.576,  0.176406,r);
case 2732: return m*s*prf2pf(4.158,  0.575,  0.164824,r);
case 2934: return m*s*prf2pf(4.218,  0.596,  0.167424,r);
case 2936: return m*s*prf2pf(4.252,  0.589,  0.169715,r);
case 3034: return m*s*prf2pf(4.285,  0.584,  0.164109,r);
case 3036: return m*s*prf2pf(4.340,  0.559,  0.165627,r);
case 3038: return m*s*prf2pf(4.393,  0.544,  0.166314,r);
case 3040: return m*s*prf2pf(4.426,  0.551,  0.167171,r);
case 3238: return m*s*prf2pf(4.442,  0.5857, 0.162741,r);
case 3240: return m*s*prf2pf(4.452,  0.5737, 0.167365,r);
case 3850: return m*s*prf2pf(4.83,   0.49611,0.168863,r);
case 3950: return m*s*prf2pf(4.76,   0.57129,0.172486,r);
case 4152: return m*s*prf2pf(4.875,  0.57329,0.16862,r);

 //Three - parameter Fermi model (3pF): liczba protonow, liczba neutronow, c=a, z=alfa, w, rho0
case 1103: return m*s*prf3pf(2.57000, 0.5052, -0.18070, 0.179227*14/11.2289,r);
case 1104: return m*s*prf3pf(2.33430, 0.4985,  0.13930, 0.165689*15/20.8773,r);
case 1212: return m*s*prf3pf(3.19230, 0.6046, -0.24920, 0.178889*24/19.4862,r);  //           norma(12,12)=19.4862
case 1213: return m*s*prf3pf(3.22500, 0.5840, -0.23634, 0.178949*25/20.5491,r);  //           norma(12,13)=20.5491
case 1414: return m*s*prf3pf(3.34090, 0.5803, -0.23390, 0.181216*28/23.298,r);   //           norma(14,14)=23.298
case 1415: return m*s*prf3pf(3.33812, 0.5472, -0.20312, 0.183256*29/24.5853,r);  //           norma(14,15)=24.5853
case 1416: return m*s*prf3pf(3.25221, 0.5532, -0.07822, 0.175686*30/29.0314,r);  //           norma(14,16)=29.0314
case 1516: return m*s*prf3pf(3.36925, 0.5826, -0.17324, 0.181196*31/27.7566,r);  //           norma(15,16)=27.7566
case 1718: return m*s*prf3pf(3.47632, 0.5995, -0.102,   0.171426*35/33.5399,r);  //           norma(17,18)=33.5399
case 1720: return m*s*prf3pf(3.55427, 0.5885, -0.132,   0.177794*37/34.4144,r);  //           norma(17,20)=34.4144
case 1920: return m*s*prf3pf(3.74325, 0.5856, -0.2012,  0.176237*39/34.2749,r);  //           norma(19,20)=34.2749
case 2020: return m*s*prf3pf(3.76623, 0.5865, -0.16123,  0.16983*40/36.259,r);   //           norma(20,20)=36.259
case 2028: return m*s*prf3pf(3.7369, 0.5245, -0.030,    0.188785*48/45.6756,r);  //           norma(20,28)=45.6756
case 2830: return m*s*prf3pf(4.3092, 0.5169, -0.1308,   0.169202*58/50.8799,r);  //           norma(28,30)=50.8799
case 2832: return m*s*prf3pf(4.4891, 0.5369, -0.2668,   0.176259*60/49.7148,r);  //           norma(28,32)=49.7148
case 2833: return m*s*prf3pf(4.4024, 0.5401, -0.1983,   0.176952*61/52.5212,r);  //           norma(28,33)=52.5212
case 2834: return m*s*prf3pf(4.4425, 0.5386, -0.2090,   0.177167*62/53.0451,r);  //           norma(28,34)=53.0451
case 2836: return m*s*prf3pf(4.5211, 0.5278, -0.2284,   0.177731*64/53.855,r);   //           norma(28,36)=53.855

 //Three - parameter Gaussian model (3pG):liczba protonow, liczba neutronow, c=a, z=alfa, w, rho0
case 1616: return m*s*prf3pg(2.549,2.19110, 0.16112,0.224317*32/32,r);       //     norma(16,16)=32
case 4050: return m*s*prf3pg(4.434,2.5283,  0.35025,0.189844*90/102.817,r);  //     norma(40,50)=102.817
case 4051: return m*s*prf3pg(4.32520,2.5813,0.43325,0.200597*91/110.438,r);  //     norma(40,51)=110.438
case 4052: return m*s*prf3pg(4.45520,2.5503, 0.33425,0.19084*92/104.035,r);  //     norma(40,52)=104.035
case 4054: return m*s*prf3pg(4.49420,2.5853,0.29625,0.189365*94/103.64,r);   //    norma(40,54)=103.64
case 4056: return m*s*prf3pg(4.50320,2.6023,0.34125,0.191673*96/109.176,r);  //     norma(40,56)=109.176
case 4250: return m*s*prf3pg(4.6110,2.527,0.1911,   0.176603*92/94.0081,r);  //     norma(42,50)=94.0081

  //Fourier-Bessel Coefficients
 case  201: return m*s*prfFB(fb201,   r)*A/a;
 case  606: return m*s*prfFB(fb606,   r)*A/a; 
 case  708: return m*s*prfFB(fb708,   r)*A/a;
 case 1618: return m*s*prfFB(fb1618,  r)*A/a;
 case 1620: return m*s*prfFB(fb1620,  r)*A/a;
 case 2228: return m*s*prfFB(fb2228,  r)*A/a;
 case 2430: return m*s*prfFB(fb2430,  r)*A/a;
 case 3242: return m*s*prfFB(fb3242,  r)*A/a;
 case 3244: return m*s*prfFB(fb3244,  r)*A/a;
 case 4252: return m*s*prfFB(fb4252,  r)*A/a;
 case 4254: return m*s*prfFB(fb4254,  r)*A/a;
 case 4256: return m*s*prfFB(fb4256,  r)*A/a;
 case 4258: return m*s*prfFB(fb4258,  r)*A/a;
 case 4658: return m*s*prfFB(fb4658,  r)*A/a;
 case 4660: return m*s*prfFB(fb4660,  r)*A/a;
 case 4662: return m*s*prfFB(fb4662,  r)*A/a;

//Sum of gaussians model
 case 102: return m*s*prfSOG(sog102, r);
 case 202: return m*s*prfSOG(sog202, r);

//charge distribution analysis
case 101: return m*s*prfMI(2.1166,0.21,0.77,r); 
//case 101: return s*prfMI(2.116,0.21,0.77,r); 
//case 202: return s*prfMI(1.67114,0.45,1.92,r); 

default:  kod=best_pn(pr, nr); // szukamy najbliższego zaimplementowanego modelu
          m=(kod/100+kod%100)/A; 
	  break; 
}}
return 0; //tu nigdy nie dojdzie 
}

  ///////////////////////////////////////////////////////////////////////////////
  /// for given p, n find the nearest known  p, n   
  ///////////////////////////////////////////////////////////////////////////////
  int anynucleus::best_pn(int p, int n)
    { 
      const int N=85;
      //tablica liczb protonow i neutronow, dla ktorych istnieja modele gestosci
      int znane[N]={0,1,100,101,102,201,202,304,405,505,506,606,607,608,708,808,809,810,
		 910,1010,1012,1103,1104,1212,1213,1214,1314,1414,1415,1416,1516,
		1616,1618,1620,1718,1720,1818,1822,1920,2020,2028,2226,2228,2328,
		2426,2428,2429,2430,2530,2628,2630,2632,2732,2830,2832,2833,2834,
		2836,2934,2936,3025,3034,3036,3038,3040,3238,3240,3242,3244,3850,
		3950,4050,4051,4052,4054,4056,4152,4250,4252,4254,4256,4258,4658,
		4660,4662};
      int besti=0;
      int best=pow2(znane[0]/100-p)+pow2(znane[0]%100-n);
      for(int i=1; i<N; i++)
        {int x=pow2(znane[i]/100-p)+pow2(znane[i]%100-n);
	 if(x==0) return znane[i];
	 if(x<best) {best=x;besti=i;}
	}
       return znane[besti]; 	
    }
  
  ///////////////////////////////////////////////////////////////////////////////
  /// random distance from the center waighted with nuclear density
  ///////////////////////////////////////////////////////////////////////////////
  double anynucleus::get_random_r()
  { double max_density=0.3/(fermi*fermi*fermi);//poprawi�!!!!!!!!!
    double r0=radius();
    double r1;
    do
      {
      r1 = r0*frandom_sqr(); // rozklad kwadratowy
      }
    while ( density(0,r1) < frandom ()*max_density*(pr+nr+1)/A());
    return r1;
  } 
  

  ///////////////////////////////////////////////////////////////////////////////
  ///  get nucleon with (random) momentum not exceeding localkf(r)
  ///////////////////////////////////////////////////////////////////////////////
  particle anynucleus::get_nucleon(vec r)
  { particle p0;
    p0.r=r;
    if(p+n==1)
    { if(p==1)
        p0.set_proton();
      else
        p0.set_neutron();
      return p0;
    }
    
    double x=frandom(); 
    if(x*(pr+nr)<pr)
        p0.set_proton();
      else
        p0.set_neutron();
    double lkf;

    if(local_kf==0)
      lkf=kf;
    else
      lkf=localkf(p0.pdg, r.length());
      
//    cout<<"lkf="<<lkf<<endl;
//    if(bodek)
//      p0.set_momentum(bodek_rand_from_ball(lkf));
//    else
      p0.set_momentum(rand_from_ball(lkf));
    return p0;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// remove nucleon P from the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void anynucleus::remove_nucleon(particle P)
  {
   switch(P.pdg)
    {case pdg_proton: pr--;break;
     case pdg_neutron: nr--;break;
     default: return;
    }
    missingp4=missingp4+P.p4();
    // cout<< "p="<<pr<< "n="<<nr<<endl;
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// insert nucleon P into the nucleus 
  ///////////////////////////////////////////////////////////////////////////////
  void anynucleus::insert_nucleon(particle P)
  { switch(P.pdg)
    {case pdg_proton: pr++;break;
     case pdg_neutron: nr++;break;
     default: return;
    }
    missingp4=missingp4-P.p4();
  }

  ///////////////////////////////////////////////////////////////////////////////
  /// check if any of p[0]...p[n-1] is Pauli blocked 
  ///////////////////////////////////////////////////////////////////////////////
    bool anynucleus::pauli_blocking (particle p[], int n) 
    {
    for(int i=0;i<n;i++)
      if(pauli_blocking(p[i]))
       {
	 //cout<<"PAULI BLOCKING"<<endl;
         return true; 
       }
    return false;
    }

  ///////////////////////////////////////////////////////////////////////////////
  /// check if particle def by pdg and momentum is blocked at dist r from center
  ///////////////////////////////////////////////////////////////////////////////
    bool anynucleus::pauli_blocking (int pdg, double p, double r)    // TRUE = PARTICLE IS BLOCKED
    {
    if(pr+nr==1 or not nucleon(pdg))
      return false;
    if(local_kf==0)
      return p<kf;
    else
      return p<localkf(pdg,r);
    }

