#ifndef _balancer_h_
#define _balancer_h_

#include "generatormt.h"
//#include "params.h"

///////////////////////////////////////////////////////////////////////////////////////////
/// 
/// The idea is to accumulate weights of different classes of events and have the balancer 
/// choose them with probabiliy proportional to the avarage weight.
/// This version differs form the chooser with the fact that the weihgt is not modified 
/// but the probability of choosing any given dynamics is reported with the method 
/// double ratio(int i)
///
///////////////////////////////////////////////////////////////////////////////////////////

template < int N > class balancer
{
private:
  double _Max;        ///< maximal value (global) 
  double _sum[N];     ///< sum of values in i-th bin 
  double _max[N];     ///< maximal value in i-th bin
  int    _count[N];   ///< number of values i i-th bin
  bool   _on[N];      ///< indicates whether the i-th process is enabled 
  double _dist[N + 1];///< probability distribution
  int    _i;          ///< numer of calls to choose_dynamics

public:
  /// conctruct based on the parameters 
  /// (some dynamics can be switched off ) 
  balancer (/*params & p*/)
  { 
    _Max=0;
    for (int i = 0; i < N; i++)
      { 
	_sum[i] = 0;
	_max[i] = 0;
	_count[i] = 0;
	_on[i] = 1;
	_dist[i+1] = _dist[i] + _on[i];
      }
    _i=0;
  }

  /// get info about the process cross section (probability)
  void add (int i, double value)
  {
    if (i >= N)
      throw *this;
    _count[i]++;
    _sum[i] += value;
    if (value > _max[i])
      {
	_max[i] = value;
	if (value > _Max)
	  _Max = value;
      }
    
     // try do make permanent updating
     for(;i<N;i++)
       _dist[i+1]=_dist[i] + ( _count[i] ? _sum[i] / _count[i] : 0);
  }

  /// choose the i-th bin with the probability proportional to ratio(i)
  int choose_bin ()	
  { 
    // at the beginning visit each of the bins 5 times to obtain
    // initial values in each bin
    if(_i<15*N)	  
      return _i++ % N;	     
    else
      if(_i++>1000) 
//        _i=0;	      
        _i=_i;
    double x = frandom () * _dist[N];
    int i = N;
    while (x < _dist[--i]);
    // here i is the such that _dist[i] <= x < _dist[i+1]
    return i;
  }

  /// make the ratios proportional to the average bin values
  /// and reset the stored values
  void update_ratios_avg ()
  {
    for (int i = 0; i < N; i++)
      {
	if (_count[i] == 0)
	  _count[i] = 1;
        _dist[i + 1] = _dist[i] + _sum[i] / _count[i];	
	_sum[i] = _count[i] = 0;	
      }
    _Max = 0;			
    for (int i = 0; i < N; i++)
      {
	if (_max[i] > 0)
	  _max[i] /= ratio (i);
	if (_max[i] > _Max)
	  _Max = _max[i];
      }
  }

  /// make the ratios proportional to the maximum bin values
  /// and reset the stored values
  void update_ratios_max ()
  {
    for (int i = 0; i < N; i++)
      {
	if (_count[i] == 0)
	  _count[i] = 1;
	_dist[i + 1] = _dist[i] + _max[i];
	_sum[i] = _count[i] = 0;
      }
    _Max = 0;
    for (int i = 0; i < N; i++)
      {
	if (_max[i] > 0)
	  _max[i] /= ratio (i);
	if (_max[i] > _Max)
	  _Max = _max[i];
      }
  }

  /// the probability of choosing i-th bin
  double ratio (int i)	
  {
    return (_dist[i + 1] - _dist[i])/_dist[N];
  }

  /// number of  values  
  double count()
  { int cnt=0;
    for(int i=0;i<N;i++)
      cnt+=_count[i];
    return cnt;
  }

 /// number of  values in i-th bin 
  double count(int i)
  {
    return _count[i];
  }

  /// sum of all values  
  double sum()
  { double s=0;
    for(int i=0;i<N;i++)
      s+=_sum[i];
    return s;
  }

  /// sum of the values in i-th bin 
  double sum(int i)
  {
    return _sum[i];
  }

  /// average value
  double avg()
  {
    return sum()/count();
  }

  /// average value in i-th bin
  double avg(int i)
  { 
    if(_count[i]==0)
      return 0;
    return _sum[i]/_count[i];
  }

  /// maximal value
  double max ()
  {
    return _Max;
  }

  /// maximal value in i-th bin 
  double max (int i)
  {
    return _max[i];
  }
};
#endif
