#ifndef _event1_h_
#define _event1_h_

#include <iostream>
#include <vector>
#include "TObject.h"
#include "particle.h"
#include "params.h"
#include "dis/LeptonMass.h"


class flags
{
public:
// primary vertex flags
  bool coh;
  bool qel;
  bool dis;
  bool res;
  bool nc;
  bool cc;
  bool anty;			// true if antineutrino 
//    int target; // 0 - free nucleon 1 - Argon 2 - Oxygen 

//double xx, yy, zz, phiphi;

// nuclear effects flags 
  //bool reint;
  //bool pauli;
  //bool rpa;
  //bool pot; // momentum dependant potential
  //double q2;
  //double W;
  //double mom;
  //double bin;
  //int number;
};

using namespace std;

class event:public TObject
{
public:
  flags flag;
  params par;
    vector < particle > in;    ///< incoming particles
    vector < particle > temp;  ///< temporary particles (daughters of primary vertex)
    vector < particle > out;   ///< outgoing particles
    vector < particle > post;  ///< results of the kaskadaevent
    vector < particle > all;   ///< all particles

    double weight;             ///< cross section

    int dyn;                   ///< dynamics code (from proctable.h)

  /// initial neutrino
  particle nu ()
  {
    return in[0];
  }
  /// initial nucleon
  particle N0 ()
  {
    return in[1];
  }
  /// fourmomentum transfer squared
  double q2 ()
  {
    vect q = in[0] - out[0];
    return q * q;
  }
  /// s viariable
  double s ()
  {
    vect q = in[0] + in[1];
    return q * q;
  }
  /// neutrino energy
  double E ()
  {
    return in[0].t;
  }
  /// number of outgoing particles
  int n ()
  {
    return out.size ();
  }				
  /// number of outgoing particles with given pdg
  int nof (int pdg)		
  {
    int c = 0;
    for (int i = 0; i < out.size (); i++)
      c += (out[i].pdg == pdg);
    return c;
  }


  int outof_cut_z (int pdg, double cutz)
  {
    int c = 0;
    for (int j = 0; j < out.size (); j++)
      {
	if (out[j].pdg == pdg && fabs (out[j].z) > cutz)
	  c++;
      }
    return c;
  }


  int outof_cut (int pdg, double cut)
  {
    int c = 0;
    for (int j = 0; j < out.size (); j++)
      {
	if (out[j].pdg == pdg && out[j].momentum () > cut)
	  c++;
      }
    return c;
  }


  double W ()
  {
    vect h = out[1];
    for (int a = 2; a < out.size (); a++)
      h = h + out[a];
    return sqrt (h * h);
  }

  double rozpr ()
  {
    return (in[0].x * out[0].x + in[0].y * out[0].y +
	    in[0].z * out[0].z) / in[0].momentum () / out[0].momentum ();
  }

  double nukrozp ()
  {
    return (in[0].x * out[1].x + in[0].y * out[1].y +
	    in[0].z * out[1].z) / in[0].momentum () / out[1].momentum ();
  }

  double nukinit ()
  {
    return (in[0].x * out[0].x + in[0].y * out[0].y +
	    in[0].z * out[0].z) / in[0].momentum () / out[0].momentum ();
  }

  double Erec ()
  {
    double M = in[1].m ();
    double m = out[0].m ();
    double Eb = par.nucleus_E_b;
    double Eprim = out[0].E ();
    double kprim = out[0].momentum ();

    return (M * Eb + M * Eprim - Eb * Eprim - Eb * Eb / 2 - m * m / 2) /
      (M - Eb - Eprim + kprim * rozpr ());
  }

  double Q2rec ()
  {
    double m = out[0].m ();
    double Eprim = out[0].E ();
    double kprim = out[0].momentum ();

    return -m * m + 2 * Erec () * (Eprim - kprim * rozpr ());
  }


  double Erec2 ()
  {
    double M = in[1].m ();
    double m = out[0].m ();
    double Eb = 49;
    double Eprim = out[0].E ();
    double kprim = out[0].momentum ();

    return (M * Eb + M * Eprim - Eb * Eprim - Eb * Eb / 2 - m * m / 2) /
      (M - Eb - Eprim + kprim * rozpr ());
  }


//double wiaz()
//{return par.nucleus_E_b  ;}
  double binding ()
  {
    double pp = in[1].momentum ();

    if (pp >= 0 && pp < 340)
      return 35.1011 + 0.00115496 * (pp - 119.571) * (pp - 119.571) +
	0.00000497491 * (pp - 95.1003) * (pp - 95.1003) * (pp - 95.1003);

    if (pp >= 340 && pp <= 800)
      return -76.5043 + 0.75004 * pp - 0.000167539 * pp * pp +
	0.000000203068 * pp * pp * pp;
  }

  double przod ()
  {
    int licz = 0;
    vect h = out[1];
    for (int a = 2; a < out.size (); a++)
      h = h + out[a];

    vect tran = in[0] - out[0];
    vec ptr = vec (tran.x, tran.y, tran.z);

    for (int b = 1; b < out.size (); b++)
      {
	if ((out[b].pdg != 111) && (out[b].pdg != 22) && (out[b].pdg != 130)
	    && (out[b].pdg != 2112) && (out[b].pdg != -2112))

	  {
	    vect hh = out[b];
	    hh.boost (-h.speed ());

	    vec phh = vec (hh.x, hh.y, hh.z);

	    if (ptr * phh > 0)
	      licz = licz + 1;
	  }
      }
    return licz;
  }

  double tyl ()
  {
    int licz = 0;
    vect h = out[1];
    for (int a = 2; a < out.size (); a++)
      h = h + out[a];

    vect tran = in[0] - out[0];
    vec ptr = vec (tran.x, tran.y, tran.z);

    for (int b = 1; b < out.size (); b++)
      {
	if ((out[b].pdg != 111) && (out[b].pdg != 22) && (out[b].pdg != 130)
	    && (out[b].pdg != 2112) && (out[b].pdg != -2112))

	  {
	    vect hh = out[b];
	    hh.boost (-h.speed ());

	    vec phh = vec (hh.x, hh.y, hh.z);

	    if (ptr * phh < 0)
	      licz = licz + 1;
	  }
      }
    return licz;
  }

public:
  event ()
  {
  }
  ClassDef (event, 1);
};

#endif
