#include <cmath>
#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include "generatormt.h"
#include "vect.h"
#include <queue>
#include "pdg.h"
#include "nucleus.h"
#include "flatnucleus.h"
#include "anynucleus.h"
#include "event1.h"
#include <TROOT.h>
#include <TTree.h>
#include "beam.h"
#include "Metropolis.h"
#include "proctable.h"
#include "nucleusmaker.h"

using namespace std;
using namespace PDG;

queue < particle > parts ;

#define LOG(x) cout<< x <<endl;
#define LOG2(x,y) cout<< x << y <<endl;
#define ERR(x) cerr<< x <<endl;
#define ERR2(x,y) cerr<< x << y <<endl;


/// true if particle p left its formation zone
bool
can_interact (particle & p)
{ return true;
  bool res = p.travelled > 0.4 * fermi;
  if (!res)
    cout << "Can't interact yet - travelled only " << p.travelled / fermi << " fm" << endl;
  return res;
}

/// process the event 
/// put reintreacition particles to tmp vector ???
/// put outgoing particles to out vector

void procinfo(particle p1, particle p2, int n, particle p[])
{

}


int
kaskadaevent (params & par, vector<particle> &input, vector<particle>& output,vector<particle> &all)
{ 
  int result=0; 
  double maxpath=0.2*fermi; 
  if(not par.kaskada_on)
    return 0; 
  nucleus* pnucleus=make_nucleus(par);
  nucleus& nucl=*pnucleus; 

  // take all nucleons and pions from tmp vector
  // others copy directly to thoutput vector
  for (int i = 0; i < input.size (); i++)
    {
      particle p = input[i];
      if (nucleon_or_pion (p.pdg))
	parts.push (p);
      else
      {
	p.endproc=escape;
	output.push_back (p);
	all.push_back(p);
      }
    }
  while (parts.size () > 0)
    {	
///// Process one particle from the queue, put results to the end of the queue 
//  cout << " There are " << parts.count () << " particles in the cascade " << endl;

      particle p1 = parts.front();
      parts.pop();

      double r = p1.r.length ();

      double d = nucl.density (0, r);
      double d0 = d * nucl.frac_neutron ();
      double d1 = d * nucl.frac_proton ();

      double s0, s1;

      Interaction ().total_cross_sections (p1.pdg, p1.Ek (),  s0, s1);

      double total = d0*s0+d1*s1;

      double freepath = -log (frandom ()) / total;

      p1.krok (min (maxpath, freepath));	// propagate by no more than maxpath
      
      if (nucl.is_outside (p1.r.length ()))
	{
	  if (nucleon (p1.pdg))	// reduce momentum when leaving nucleus
	    p1.set_momentum (p1.p () * (1 - nucl.V() / p1.E ()));
	  p1.endproc=escape;
	  output.push_back (p1);
	  all.push_back (p1);
	}
      else if (not can_interact (p1) or maxpath < freepath)	// no interaction during maxpath
	    {
	      parts.push (p1);	// put to end of queue for further processing
	    }
	  else	// can_interact(p1) and freepath < maxpath   (i.e. there was interaction )
	    {
////	      cout << "Interaction: kinematics" << endl;
	      particle p2;	// nucleon from nucleus 
	      particle p[4];	// place for the result of scatering
	      int n = 2;	// place for the number of particles after scattering
	      vect js (0, 0, 0, 0);	// check energy-momentum conservation 
	      Interaction I; 
	      int res;
	      do
		{
		  res = I.particle_scattering (p1, p2, s1*d1 / total, n, nucl, p);
		}
	      while (res == 0);
//CJ	      cout<<" Interaction: "<<I.process_name()<<" ("<<I.process_id()<<") ";
	      for (int i = 0; i < n; i++)
		if (!p[i].is_valid ())
		  {
////		    cout << "Interaction: error" << p[i] << endl;
                    delete pnucleus;
		    return 0;
		  }
	      if ((res == 0) or nucl.pauli_blocking (p, n))	// some particle was Pauli blocked
		{
//CJ		  cout << ": Pauli blocked." << endl;
		  parts.push (p1);	// so nothing happened - put p1 to end of queue
		}
	      else
		{// Scattering happened  
		  result=1;
//CJ		  cout << ": completed." << endl;
		  p1.endproc=I.process_id();
		  all.push_back(p1);
		  nucl.remove_nucleon (p2);
////		  cout << "Interaction: " << n << "particle added to kaksda" << endl;
		  for (int i = 0; i < n; i++)
		    {
		      p[i].travelled = 0;
		      js = js + p[i];
		      if (nucleon (p[i].pdg) and p[i].Ek () < nucl.V())	// to low energy to leave nucleus
			{
////			  cout << "Ek=" << p[i].Ek () << "  t.V=" << nucl.  V << endl;
			  // nucleon trapped - only seen as nucleus excitation
			  p[i].endproc=jailed;
			  all.push_back(p[i]);
			  nucl.insert_nucleon (p[i]);
			}
		      else
			parts.push (p[i]);	// queue for further processing
		    }
		  procinfo(p1,p2,n,p);
////		  cout << p1 + p2 << "===?===" << js << "  p2=" << p2 << endl;
		}// scattering ended
	    }
	
    } // queue parts is empty here

  delete pnucleus;
  return result;

};
