#include <fstream>
#include <sstream>
#include "kaskada7.h"
#include "beam.h"
#include "flatnucleus.h"
#include "anynucleus.h"
#include <TFile.h>
#include <TTree.h>
#include "args.h"
#include "nucleusmaker.h"

using namespace std;

int
main (int argc, const char **argv)
{
  args a("kaskadapions","kaskadapions.txt","kaskadapions.root");
  a.read (argc, argv);
  params p;
  p.read (a.input);
  p.read (a.params, "Command line");
  p.list ();
  frandom_init(p.random_seed);

  nucleus *pnucleus = make_nucleus (p);
  nucleus & jadro = *pnucleus;

  event *e = new event;
  TFile *f = new TFile (a.output, "recreate");
  TTree *t2 = new TTree ("treeout", "Tree of events");

  t2->Branch ("e", "event", &e);	///< tree1 has only one branch (branch of events)

  string name[2] = { "flat", "real" };

  stringstream filename1,filename2,filename3,filename4;
  filename1 << "pions-reaction-" << p.nucleus_p << "-" << name[p.nucleus_model] << ".dat" << flush;
  filename2 << "pions-absorption-" << p.nucleus_p << "-" << name[p.nucleus_model] << ".dat" << flush;
  filename3 << "pions-cex-" << p.nucleus_p << "-" << name[p.nucleus_model] << ".dat" << flush;
  filename4 << "pions-inelastic-" << p.nucleus_p << "-" << name[p.nucleus_model] << ".dat" << flush;

  ofstream plik1 (filename1.str ().c_str ());
  ofstream plik2 (filename2.str ().c_str ());
  ofstream plik3 (filename3.str ().c_str ());
  ofstream plik4 (filename4.str ().c_str ());

  beam b (p);

  double radius = jadro.radius ();
  double pi = M_PI;
  double przel = 10.0 * pi * radius * radius / (double) p.number_of_events / fermi / fermi;	//na mb
  srand (time (0));
  vec position;
  int N1, N2, N3, N4, N5;

  for (int energy = 200; energy < 500; energy += 10)
    {
      N1 = 0;
      N2 = 0;
      N3 = 0;
      N4 = 0;
      N5 = 0;

      for (int i = 0; i < p.number_of_events; i++)
	{
	  double x, y, z;
	  do
	    {
	      x = 2 * frandom () - 1;
	      y = 2 * frandom () - 1;
	    }
	  while (x * x + y * y > 1);

	  z = -sqrt (1 - x * x - y * y);

	  cout << "                         \rE="<<energy<<"  event=" << i << flush; 
	  e = new event;
	  particle p0 = b.shoot ();
	  p0.r =0.999* vec (x, y, z) * jadro.radius ();
	  p0.set_energy ((double) energy);

	  e->out.push_back (p0);

          int res= kaskadaevent (p, *e);
           
	  int n1 = 0, n2 = 0, n3 = 0;

	  for (int j = 0; j < e->post.size (); j++)
	    {
	      switch (e->post[j].pdg)
		{
		case 211:  n1++;  break;
		case -211: n2++;  break;
		case 111:  n3++;  break;
		}
	    }

	  if (res == 1){ 
	    N1++  ;// there was interaction
	  if (n1 == 0 && n2 == 0 && n3 == 0)
	    N2++;//absorption
	  else if (n1 == 0 && n2 == 1 && n3 == 0)
	    N3++;//pi-
	  else if (n1 == 0 && n2 == 0 && n3 == 1)
	    N4++;//pi0
	  else
	    N5++;//inelastic
	  }
	  
	  t2->Fill ();
	  delete e;
	  cout << " completed.\r"<< flush ;

	}
      plik1 << energy - 140 << " " << (double) N1 * przel << endl;
      plik2 << energy - 140 << " " << (double) N2 *przel << endl;
      plik3 << energy - 140 << " " << (double) (N3 + N4) * przel << endl;
      plik4 << energy - 140 << " " << (double) N5 *przel << endl;
    }

  f->Write ();

  plik1.close ();
  plik2.close ();
  plik3.close ();
  plik4.close ();

  delete t2;
  delete f;
  genrand_write_state();
  return 0;
}
