#ifndef _nucleus_h_
#define _nucleus_h_

#include "particle.h"
#include "pdg.h"
#include "params.h"
#include "generatormt.h"

//#include <algorithm>  

using namespace std;

/// this class implements the target nucleus
/// it should take into account of the its radius, charge ,
/// proton and neutron density as well as local fermi momentum and Pauli blocking
/// the number of nucleons can change as a result of scattering and the density 
/// and Fermi momentum should be consistently updated - this is still NOT implemented

class nucleus  // abstract Nucleus class
{
protected:
  int p;  //number of protons
  int n;  //number of neutrons
public:
  nucleus (params & par):p(par.nucleus_p),n(par.nucleus_n){}
  virtual int  A(){return n+p;}                   ///< A
  virtual int  Z(){return p;}                     ///< Z
  virtual int  N(){return n;}                     ///< N
  virtual double radius()=0;			  ///< end of nucleus
  virtual double V()=0;				  ///< bounding energy
  virtual double proton_density (double r)=0;     ///< proton_density 
  virtual double neutron_density (double r)=0;    ///< neutron_density 
  virtual double frac_proton ()                   ///< percentage of protons 
  { return double (p) / (p + n); 
  }
  virtual double frac_neutron ()                  ///< percentage of neutrons
  { return double (n) / (p + n); 
  }
  virtual bool is_outside (double r)              ///< r reaches outside the nucleus
  {return r>radius();
  }
  virtual double density (int pdg = 0, double r = 0)=0; ///<proton/neutron density at dist r from center
  virtual double get_random_r ()=0;               ///< random distance from the center 
  virtual double localkf (int pdg, double r)=0;	  ///< local Fermi momentum
  virtual double kf_from_density (double dens)=0; ///< local Fermi moementum from local density
  virtual particle shoot (vec r)=0;	          ///< choose particle and its momentum 
  virtual particle shoot ()=0;		          ///< choose particle its momentum and position
  virtual particle get_nucleon (vec r)=0;         ///< choose particle its momentum and position
  virtual void remove_nucleon (particle P)=0;	  ///< remove nucleon from the nucleous
  virtual void insert_nucleon (particle P)=0;	  ///< insert nucleon back to the nucleous
  virtual bool pauli_blocking (particle & pa)=0;	// TRUE = PARTICLE IS BLOCKED
  virtual bool pauli_blocking (particle p[], int n)=0;	// true = any of the particles is blocked
  virtual bool pauli_blocking (int pdg, double p, double r)=0;	// true = particle is blocked
  virtual double mstar (double r)=0;	          ///< nucleon effective mass inside nucleus
};

#endif
