#include <iostream>
#include <iomanip>
#include <sstream>
#include <vector>
#include "event1.h"
#include "TROOT.h"
#include "TTree.h"
#include "TFile.h"
#include "qelevent.h"
#include "pdg.h"
#include "chooser.h"
#include "beam.h"
#include "target.h"
#include "dis/disweight.h"
#include "dis/dishadr.h"
#include "TMCParticle.h"
#include <TPythia6.h>
#include "stdlib.h"
#include "dis/singlepion.h"
#include "dis/charge.h"
#include "dis/resweight.h"
#include "dis/reshadr.h"
#include "pauli.h"
#include "dis/disevent.h"
#include "dis/resevent.h"
#include "dis/resevent2.h"
#include "cohevent2.h"
#include "args.h"
#include "kaskada7.h"

extern double SPP[2][2][2][3][40];
extern double sppweight;

params p;

//////////////////////////////////////////////
//   Parameters used in the coherent part:
//     int cohA;
//     int cohA2;
//////////////////////////////////////////////

void makeevent(event* e, params &p, target & nucleus, int dyn)
{    
      e->weight=0;

      switch (dyn)
	{
	case 0:
	  if (p.dyn_qel_cc)
	    {
	      qelevent (p, *e, nucleus, false);
	      if (p.pauli_blocking)
		  mypauli_qel (*e, nucleus);
	    }
	  break;		// qel cc
	case 1:
	  if (p.dyn_qel_nc)
	    {
	      qelevent (p, *e, nucleus, true);
	      if (p.pauli_blocking)
		  mypauli_qel (*e, nucleus);
	    }
	  break;		// qel nc
	case 2:
	  if (p.dyn_res_cc)
	    {
	      resevent2 (p, *e, true);
	      if (p.pauli_blocking)
		  mypauli_spp (*e, nucleus);
	    }
	  break;		//true for dis cc
	case 3:
	  if (p.dyn_res_nc)
	    {
	      resevent2 (p, *e, false);
	      if (p.pauli_blocking)
		  mypauli_spp (*e, nucleus);
	    }
	  break;
	case 4:
	  if (p.dyn_dis_cc)
	    {
	      disevent (p, *e, true);
	      if (p.pauli_blocking)
		  mypauli_spp (*e, nucleus);
	    }
	  break;		//true for dis cc
	case 5:
	  if (p.dyn_dis_nc)
	    {
	      disevent (p, *e, false);
	      if (p.pauli_blocking)
		  mypauli_spp (*e, nucleus);
	    }
	  break;
	case 6:
	  if (p.dyn_coh_cc)
	    {
	      cohevent2 (p, *e, nucleus, true);
	    }
	  break;
	case 7:
	  if (p.dyn_coh_nc)
	    {
	      cohevent2 (p, *e, nucleus, false);
	    }
	  break;
	}

        e->dyn = dyn;

        if (e->weight == 0)
          {
            e->out.clear ();
            e->out.push_back (e->in[0]);	//po co to wpisywanie ???
            e->out.push_back (e->in[1]);
          }
      
        e->flag.cc  = dyn == 0 || dyn == 2 || dyn == 4 || dyn == 6;
        e->flag.nc  = dyn == 1 || dyn == 3 || dyn == 5 || dyn == 7;
      
        e->flag.qel = dyn == 0 || dyn == 1;
        e->flag.res = dyn == 2 || dyn == 3;
        e->flag.dis = dyn == 4 || dyn == 5;
        e->flag.coh = dyn == 6 || dyn == 7;

}// end of makeevent


int main (int argc, const char **argv)
{
  args a;
  a.read (argc, argv);
  p.read (a.input);
  p.read (a.params, "command line");
  p.list ();
  frandom_init(p.random_seed);

//  if(p.dyn_dis_nc or p.dyn_res_nc  or p.dyn_dis_cc or p.dyn_res_cc )
  {
  singlepion (p);
  for (int c = 0; c < 40; c++)
    cout << 1210 + c * 20 << "  " 
         << SPP[0][0][0][0][c] << "  " 
	 << SPP[0][0][0][1][c] << "  "
         << SPP[0][0][0][2][c] << "  " 
	 << SPP[0][0][1][0][c] << "  " 
	 << SPP[0][0][1][1][c] << "  " 
	 << SPP[0][0][1][2][c] << endl;
  }
  event *e = new event;

  TFile *ff = new TFile (a.output, "recreate");
  TTree *tf = new TTree ("treeout", "Drzewo zdarze�");
  tf->Branch ("e", "event", &e);

  delete e;

  target nucleus (p);
  beam neutrino_beam (p);


//////////////////////////////////////////////////////////////
//              Test events
////////////////////////////////////////////////////////////// 

  const int NPROC = 8;
  chooser < NPROC > procesy (p);
  
  for (int i = 0; i < p.number_of_test_events; i++)
    {
      int dynamika = procesy.choose ();	// choose dynamics 

       e = new event ();

       e->in.push_back (neutrino_beam.shoot());	// insert neautrino
       e->in.push_back (nucleus.shoot ());	// insert target nucleon

       makeevent(e,p,nucleus,dynamika);

       procesy.add (dynamika, e->weight);

       delete e;
     }
//////////////////////////////////////////////////////////////
//              end of test events
////////////////////////////////////////////////////////////// 

  cout <<"dyn   ratio  max_value mean_value" <<  endl;
  for (int j = 0; j < NPROC; j++)
    cout << procesy.weight (j) << "  " << procesy.
      ratio (j) << "  " << procesy.max (j) <<" "<< procesy.avg (j) << endl;

  procesy.update_weights_avg ();

// calculate desired number of events for each dynamics
  int desired[NPROC];
  int ilosc = p.number_of_events;
  double total=1.0;
  for (int k = 0; k < NPROC-1; k++)
    { 
      double frac=procesy.ratio(k);
      if(frac==0)
	desired[k]=0;
      else
	desired[k] = int(frac/total*ilosc+0.5 );
      ilosc-=desired[k];
      total-=frac;
    }
  desired[NPROC-1]=ilosc;


  cout <<"dyn  n   ratio     sigma " <<  endl;
  for (int k = 0; k < NPROC; k++)
    {
      cout << k <<" "<< setw(5)  << desired[k] 
	        <<" "<<  setw(10) << procesy.ratio(k)
	        <<" "<<  setw(10) << procesy.weight(k) <<  endl;
    }

/////////////////////////////////////////////////////////////
//              The main loop in NPROC
//////////////////////////////////////////////////////////////

  char filename[130];

  int savedevents = 0;
  double removed[NPROC]; // number of events to skip (delete)

  for (int k = 0; k < NPROC; k++)
    {
      double maxmax = 0;
      removed[k] = 0;

      sprintf(filename,"%s.%d.part",a.output,k);

      e = new event ();

      TFile *f1 = new TFile (filename, "recreate");
      TTree *t1 = new TTree ("treeout", "Drzewo zdarze�");

      t1->Branch ("e", "event", &e);

      delete e;

      for (int i = 0; i < desired[k] + removed[k]; i++)
	{
	  e = new event ();
	  e->in.push_back (neutrino_beam.shoot());	// insert neautrino
	  e->in.push_back (nucleus.shoot ());	// insert target nucleon
          e->in[0].r=e->in[1].r;

          makeevent(e,p,nucleus,k);

	  for(int j=0;j<e->out.size();j++)
	    e->out[j].r=e->in[1].r;

	  double wagha = e->weight;

	  if (wagha > maxmax * frandom ())
	    {
	      for(int j=0;j<e->in.size();j++)
		{particle p=e->in[j];
		 p.endproc=k;
		 e->all.push_back(p);
		} 
       	      kaskadaevent(p,e->out,e->post,e->all);  // runs only if p.kaskada_on is true
	      t1->Fill ();
	      int ready =int( savedevents + i - removed[k]);
	      if (ready % 10 == 0)
		cout << "Dyn [" << k << "].  " << ready  << " events ready.\r";// << endl;
	    }
	  else
	    {
	      i--;
	    }

	  if (wagha > maxmax)
	    {
              double toremove = (wagha - maxmax)/wagha * (i-removed[k]);
	      removed[k] += toremove;
	      cout.precision(6);
	      cout << "Dyn[" << k << "]  New max =" << setw(11)<< wagha 
		   <<" Prev max=" << setw(11)<< maxmax 
		   <<"   " << toremove  << " events deleted." << endl;
	      maxmax = wagha;
	    }

	  delete e;

	}

      f1->Write ();

// elimination of spurious events foir dynamics k 
// by copying only last desired[k] events to outfile 

      int nn = t1->GetEntries ();

//      for (int jj = ceil(removed[k]); jj < nn; jj++)
      for (int jj = nn-desired[k]; jj < nn; jj++)
	{
	  e = new event();
	  t1->GetEntry (jj);
	  tf->Fill ();
	  delete e;
	}

      f1->Close ();
      delete f1;
      unlink(filename);
      savedevents += desired[k];
    }
//////////////////////////////////////////////////////////////////////////////////////
//                    end of the main loop in NPROC
//////////////////////////////////////////////////////////////////////////////////////


  ff->Write ();
  ff->Close ();
  delete ff;
  cout << "Plik wynikowy: \"" << a.output << "\"" << endl;
  genrand_write_state();
}

