#ifndef _params_h_
#define _params_h_
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <fstream>
#include "vec.h"

// all the parameters (type, name ,default_value) are defined here
#define PARAMS_ALL()\
        PARAM(int, random_seed, 1)\
        PARAM(int, number_of_events, 10000)\
        PARAM(int, number_of_test_events,10000)\
        PARAM(string, beam_energy,"1000")\
        PARAM(int, beam_particle, 14)\
        PARAM(vec, beam_direction, vec(0,0,1))\
        PARAM(int, beam_placement, 1)\
        PARAM(int, nucleus_p, 0)\
        PARAM(int, nucleus_n, 0)\
        PARAM(string,nucleus_density,"0") \
        PARAM(double,nucleus_E_b,0)\
        PARAM(double,nucleus_kf,0)\
	PARAM(int, nucleus_target,0)\
        PARAM(int, nucleus_model,0)\
        PARAM(bool, dyn_qel_cc,0)\
        PARAM(bool, dyn_qel_nc,0)\
        PARAM(bool, dyn_res_cc,0)\
        PARAM(bool, dyn_res_nc,0)\
        PARAM(bool, dyn_dis_cc,0)\
        PARAM(bool, dyn_dis_nc,0)\
        PARAM(bool, dyn_coh_cc,0)\
        PARAM(bool, dyn_coh_nc,0)\
 /*       PARAM(int, qel_kinematics,0)\
*/        PARAM(double, qel_cc_axial_mass,0)\
        PARAM(double, qel_nc_axial_mass,0)\
        PARAM(int, delta_FF_set,0)\
        PARAM(double, res_dis_cut,0)\
        PARAM(int, spp_precision,0)\
/*        PARAM(bool, qel_cc_japan,0)\
        PARAM(bool, qel_relati,0)\
*/        PARAM(bool, coh_mass_correction,0)\
        PARAM(bool, kaskada_on,1)\
        PARAM(bool, pauli_blocking,1)


using namespace std;


template <class T>
inline bool read(T &x, stringstream & s)
{
  s >>x;
  return true;
}

inline bool read(vec &a, stringstream & s)
{
  s >>a.x>>a.y>>a.z;   
  return true;
}

inline bool read( string &x , stringstream & s)
{
  getline(s,x) ;
  return true;
}


/// the params class is used to input the run time parameters
/// to the generator from a file and to provide them to the  
/// interested objects in the generator
/// it also helps to store them in the output file for the record
/// the type of the paremeter must be string, vector or  
/// any type with friend operator>>(istream&,type&) operator defined 
/// all the parameters (type, name  are defined here
class params
{//public fields
public:
/// parameter definitions
#define PARAM(type,name,default_value) type name;
PARAMS_ALL() 
#undef PARAM

public:
/// constructor
params()
{
#define PARAM(type,name,default_value) name=default_value;
PARAMS_ALL() 
#undef PARAM
}

/// read all parameter values from config file filename
inline bool read (const char *filename)
{
    ifstream dane;
    dane.open (filename);
    if (!dane)
      cerr << "Could not open: '" << filename << "' using defaults " << endl;
    return  read(dane,filename);
}

inline bool read (istream& dane,const char *filename)
{
    int line_number = 0;
    string varname, line;
    char eq;
    double value;
    while (dane.good ()) // while not end of file
      {	line_number++;   // increase line number 
	getline (dane, line);  //read next line to buffer "line" 
	stringstream ins (line); // create istream for reading from "line" 
	if (line[0] == '#')      // skip to next line if this line is commented
	  continue;
	ins >> ws;
	if (ins.peek () == '#')  // comments nust start at the beginning of the line  
	  {
	    cerr << filename << ':' << line_number <<
	      ": '#' in the middle of the line " << endl;
	    continue;
	  }
	if (ins.eof ()) // the line was empty
	  continue;
	varname = "";  // erase the name of the variable from the previous line
	eq = ' ';
	while (!ins.eof () && ins.peek () != '=' && ins.peek () != ' '
	       && ins.peek () != '\t')
	  varname += ins.get (); // read up to the next ' ', '\t', '=' or end of line
	ins >> eq;  // read the '=' sign
	if(eq!='=')   
	    {cerr << filename << ':' << line_number <<
	      ": '=' expected after  \"" <<varname<<"\""<< endl;
	    continue;
	    }
// check if the varname is in the fieldlist and call appiopriate read function
#define PARAM(type,name,default_value) ( varname == #name ? ::read(name,ins) : false) ||
    PARAMS_ALL() 
	cerr << filename << ':' << line_number <<": unknown parameter \""<<varname<<"\""<<endl; 
#undef PARAM
      }
  }

void list()
{
#define PARAM(type,name,default_value) cout<< #name" = "<< name<<endl; 
    PARAMS_ALL();
#undef PARAM
}

};


#endif
