#include "particle.h"
#define echo(x)

/// particle decay;
/// p4 is totalfourmomentum 
/// p1, p2 particles to decay into (their masses must be set beforehand)
bool
decay (vect p4, particle & p1, particle & p2)
{
  double MM = p4 * p4;
  double m = sqrt (MM);
  if (p1.mass () + p2.mass () > m)
    return false;
  // random vector form the unit ball 
  double xx, yy, zz, rr;
  do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
    }
  while ((rr = xx * xx + yy * yy + zz * zz) > 1);
  // value of momentum of decay products in cms frame
  double m1m1 = p1.mass () * p1.mass ();
  double m2m2 = p2.mass () * p2.mass ();
  double pp =
    (MM * MM + m1m1 * m1m1 + m2m2 * m2m2 -
     2 * (MM * m1m1 + MM * m2m2 + m1m1 * m2m2)) / (4 * MM);
  double k = sqrt (pp / rr);
  // adjust the length of random vector to the calculated value
  vec p (k * xx, k * yy, k * zz);
//     cout<<"p="<<p<<endl;
  p1.set_momentum (p);
  p2.set_momentum (-p);
  vec cmsspeed = p4.v ();
  // go back to the lab frame - in which initial particle has velocity p4.v();
  p1.boost (cmsspeed);
  p2.boost (cmsspeed);
  return true;
}

///  speed of a in the frame of b
double
relative_speed (particle & a, particle & b)
{
  vect plab = a.p4 ();
  plab.boost (-b.v ());
  return plab.v ().length ();
}

/// energy of a in the frame of b
double
relative_energy (particle & a, particle & b)
{
  vect plab = a.p4 ();
  plab.boost (-b.v ());
  return plab.t;	// t is energy	
}

///  
double
get_cos (double a, double b)
{
  double max = a + fabs (b) + 1;
  double x;
  do
    x = -1 + 2 * frandom ();
  while ((a * x + b) * x * x * x + 1 < max * frandom ());
  return x;
}
