#include "qelcc.h"
#include <cstdlib>

using namespace std;

/////////////////////////////////////////////////////////////////////////////////////
//                 semielastic neutrino nucleon scattering 
//                       I M P L E M E N T A T I O N
/////////////////////////////////////////////////////////////////////////////////////


/////////////////////////////////////////////////////////////////////////////////////
CC::CC (char* lepton, bool use_lambda2, double axial_mass, double hadron_mass)
/////////////////////////////////////////////////////////////////////////////////////
       :  
         Ma (axial_mass),
         _use_lambda (use_lambda2), 
         M (hadron_mass), 
         MM (hadron_mass*hadron_mass),
	 nowa_parametryzacja(false),
	 dziwny(false)
  {
    set_lepton (lepton);
  }

/////////////////////////////////////////////////////////////////////////////////////
CC::CC (double lepton_mass0, int lepton_charge, bool use_lambda2, double axial_mass, double hadron_mass)
/////////////////////////////////////////////////////////////////////////////////////
       :  
         Ma (axial_mass),
         _use_lambda (use_lambda2), 
         M (hadron_mass), 
         MM (hadron_mass*hadron_mass),
	 nowa_parametryzacja(false),
	 dziwny(false)
  {
    m=lepton_mass0;
    znak=lepton_charge;
  }
  
///s//////////////////////////////////////////////////////////////////////////////////
void CC::set_lepton (char * lepton)
/////////////////////////////////////////////////////////////////////////////////////
  {
    const char *error = "lepton must be one of: e+ e- m+ m- t+ t-";
    switch (lepton[0])
      {
      case 'e': m = m_e; break;
      case 'm':	m = m_mu; break;
      case 't':	m = m_tau; break;
      default: cerr << error << endl;
	       throw error;
      }
    switch (lepton[1])
      {
      case '+':  znak =  1; break;
      case '-':  znak = -1; break;
      default:  cerr << error << endl;
	        throw error;
      }	
    mm = m * m;
    lepton_name = lepton;
  }

/////////////////////////////////////////////////////////////////////////////////////
void CC::info (ostream & o)
/////////////////////////////////////////////////////////////////////////////////////
  {
    o << "# Semielastic " << (znak <
			      0 ? "neutrino" : "anti-neutrino") <<
      " nucleon scattering" << endl;
    o << "# lepton name    =" << lepton_name << endl;
    o << "# Neutrino Energy =" << Enu << endl;
    o << "# Jap. correction " << (_use_lambda ? "used" : "not used") << endl;
    o << "# lepton mass =" << m << endl;
    o << "#          Ma =" << Ma << endl;
  }

/////////////////////////////////////////////////////////////////////////////////////
double CC::sigma (double q2)
/////////////////////////////////////////////////////////////////////////////////////
  { 
//extern double qelmlep;
	double ksi=3.706;
    double lam2;
    _use_lambda=false;
    if (!_use_lambda)
      lam2 = 1;
    else
      {
	double k = sqrt (fabs (q2)) / GeV;
	lam2 = 1 - (0.053 + 0.017 * k) * sin (4 * k / (1 + 0.22 * k));
      }
    double Gve = 1 / kwad (1 - q2 / 0.71 / GeV / GeV);
    double Gvm = (1 + ksi) * Gve;
    
    // Nowa parametryzacja:
    if(nowa_parametryzacja)
    { if(-q2 > 7* GeV2)
         throw(" Q2 > 7 GeV2 poza zakresem poprawnoci fitu");
    double Q= sqrt(-q2)/GeV;
    double tau=-q2/(4*MM);
    double mu_p=2.793;  // moment dipolowy protonu
    double mu_n=-1.913; // moment dipolowy neutronu

    double Gep = 1/(1+Q*(0.62+Q*(0.68+Q*(2.80+Q*0.83))));

    double Gmp = mu_p/(1+Q*(0.35+Q*(2.44+Q*(0.50+Q*(1.04+Q*0.34)))));

    double Gmn = mu_n/(1+Q*(-0.74+Q*(9.29+Q*(-7.63+Q*4.63))));

    double a=1.25;
    double b=10.4;
    double GD=1/kwad(1-q2/(0.71*GeV2));
    double Gen = -a *mu_n*tau*GD/(1+b*tau);
     
    if (dziwny)
      Gep=Gmp/mu_p *(1-0.14*(-q2/GeV2-0.3)); 





//BBBA05

/*    Gep =(1-0.0577*tau)/(1+11.2*tau+13.6*tau*tau+33*tau*tau*tau);
    Gmp =2.79*(1+0.15*tau)/(1+11.1*tau+19.6*tau*tau+7.54*tau*tau*tau);
    Gen =(1.38*tau-0.214*tau*tau)/(1+8.51*tau+59.9*tau*tau+13.6*tau*tau*tau+2.57*tau*tau*tau*tau);
    Gmn =-1.91*(1+1.82*tau)/(1+14.1*tau+20.7*tau*tau+69.7*tau*tau*tau);
*/		
		
		

    
    Gve=Gep-Gen;
    Gvm=Gmp-Gmn;
    }

    double x = q2 / MM;

    // Form Faktory
    double F1v = lam2 * (Gve - Gvm * x / 4) / (1 - x / 4);
    double F2v = lam2 * (Gvm - Gve) / (1 - x / 4);
    double Fa = -1.263 / kwad (1 - q2 / Ma / Ma);
    
    double m_pi=139.57*MeV; 
    
    double Fp = 2*MM*Fa/(m_pi*m_pi-q2);
	mm=105*105;
    double A =
      (mm - q2) / (4 * MM) * ((4 - x) * Fa * Fa
			      - (4 + x) * F1v * F1v
			      - x * kwad (F2v) * (1 + x / 4)
			      - 4 * x * F1v * F2v
			      - mm / MM * (kwad (F1v + F2v) +
					           kwad (Fa + 2 * Fp) +
					           (x-4)*kwad (Fp)
                               )
                );


    double B = -x * Fa * (F1v + F2v);

    double CC = (kwad (Fa) + kwad (F1v) - x * kwad (F2v / 2)) / 4;

//  s - u
    double su = 4 * M * Enu + q2 - mm;
    double suM2 = su / MM;
    double ABC = A + znak * B * suM2 + CC * suM2 * suM2;
//cout<<A<<"   "<<znak*B*suM2<<"  "<<CC * suM2 * suM2<<endl;
//cout<<CC<<"  "<<suM2<<endl;
//cout<<su<<"  "<<MM<<endl;
//cout<<M<<" "<<Enu<<" "<<q2<<" "<<mm<<endl;
//cout<<"CRCC="<<MM * G * G * cos2thetac / 8 / Pi / Enu / Enu * ABC<<endl;;
	//cout<<"Jestem w CC.cc; E oraz q2 to  "<<Enu<<"  "<<q2<<endl;
	//cout<<"jestem w CC.cc; oto Xsec:  "<<MM * G * G * cos2thetac / 8 / Pi / Enu / Enu * ABC<<endl;
    return  MM * G * G * cos2thetac / 8 / Pi / Enu / Enu * ABC;
	
 }


// granice cakowania po q^2
// factor=-1  -dolna granica
// factor=+1  -g�na granica
/////////////////////////////////////////////////////////////////////////////////////
double CC::qq (double factor)
/////////////////////////////////////////////////////////////////////////////////////
  {
    double x;
    x = kwad (2 * Enu * M - mm) - 4 * mm * MM;
    if (x < 0)
      x = 0;
    return -fabs (mm
		  + factor * Enu / (M + 2 * Enu) * sqrt (x)
		  - Enu * (2 * Enu * M + mm) / (M + 2 * Enu));
  }
  


//granice cakowania po p�zie leptonu wg Czarka 
/////////////////////////////////////////////////////////////////////////////////////
  void CC::granice (double kplusp, double EplusEp, double &kprimmin, double &kprimmax)
/////////////////////////////////////////////////////////////////////////////////////
  {
    double ss = EplusEp * EplusEp - kplusp * kplusp;
    double temp = ss + MM - mm;
    double pprim2 = temp * temp / 4 / ss - MM;
    //pprim2=(temp*temp-4*ss*MM)/4*ss;
    if (pprim2 < 0)
      {
	kprimmin = 0;
	kprimmax = 0;
	return;
      }
    double pprim = sqrt (pprim2);
    double Ekprim = sqrt (mm + pprim2);
    double v = kplusp / EplusEp;
    double mianownik = sqrt (1 - v * v);
    kprimmin = fabs ((pprim - v * Ekprim) / mianownik);
//  cout<<"s="<<ss<<" temp="<<temp<<" k'^2="<<pprim2<<" Ek'="<<Ekprim<<" v="<<v;
//  cout<<" mian="<<mianownik;
//  cout<<" k'min="<<kprimmin;
    kprimmax = (pprim + v * Ekprim) / mianownik;
//  cout<<"k'max="<<kprimmax<<endl;
  }


