/////////////////////////////////////////////////////////////////////////////////////
// caa informacja o kwazielastycznym rozpraszaniu neutrin na nukleonach 
// na podstawie Llewelyn-Smith 
// (C) C. Juszczak 2001
/////////////////////////////////////////////////////////////////////////////////////

#ifndef _CC_h_
#define _CC_h_

#include <iostream>
#include <cmath>
#include "jednostki.h"
#include "masses.h"
#include "calga.h"

//extern double qelmlep;


bool const nowa_parametryzacja=true;
bool const dziwny=false;

inline double min (double a, double b) {  return (a < b ? a : b);}
inline double max (double a, double b) {  return (a > b ? a : b);}
inline double kwad (double x)          {  return x * x;}

/// semielastic neutrino nucleon scattering 
class CC
{
  double Enu;
  double Ma;
  bool _use_lambda;
  int znak;			//-1 gry neutrino 1 gdy antyneutrino = adunek odpowiedniego leptonu 
  double m, mm, M, MM; 
  char *lepton_name;
  bool nowa_parametryzacja;
  bool dziwny;
  
public:

   CC (char * lepton="e-",
       bool use_lambda2 = true,
       double axial_mass = 1050 * MeV,
       double hadron_mass = M12);

   CC (double lepton_mass0, 
       int lepton_charge0,
       bool use_lambda2 = true,
       double axial_mass = 1050 * MeV,
       double hadron_mass = M12);

inline  double lepton_mass ();
inline  CC& set_energy (double new_E);
inline  double energy ();
inline  CC& set_axial_mass (double axial_mass);
inline  double axial_mass ();
inline  void dont_use_lambda ();
inline  void use_lambda ();
inline  bool lambda_used ();
inline  char *lepton ();
        void set_lepton (char *lepton);
inline  void set_hadron_mass (double hadron_mass);
inline  double hadron_mass ();
        void info (std::ostream & o);
        double sigma (double q2);
inline  double Enu0 ();                               
        double qq (double factor);                  // granice cakowania
inline  double kprimmax2 (double Ek, double Ep, double pfermi);


//granice cakowania wg Czarka 
void granice (double kplusp, double EplusEp, double &kprimmin, double &kprimmax);

inline   double total_sigma(double E);

friend void porownanie();

}; //koniec klasy CC


/////////////////////////////////////////////////////////////////////////////////////
///                      I M P L E M E N T A C J A
/////////////////////////////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////////////////
double CC::lepton_mass ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return m;
  }

/////////////////////////////////////////////////////////////////////////////////////
  CC& CC::set_energy (double new_E)
/////////////////////////////////////////////////////////////////////////////////////
  {
    Enu = new_E;
    return *this;
  }

/////////////////////////////////////////////////////////////////////////////////////
  double CC::energy ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return Enu;
  }

/////////////////////////////////////////////////////////////////////////////////////
  CC& CC::set_axial_mass (double axial_mass)
/////////////////////////////////////////////////////////////////////////////////////
  {
    Ma = axial_mass;
    return *this;
  }

/////////////////////////////////////////////////////////////////////////////////////
  double CC::axial_mass ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return Ma;
  }

/////////////////////////////////////////////////////////////////////////////////////
  void CC::dont_use_lambda ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    _use_lambda = false;
  }

/////////////////////////////////////////////////////////////////////////////////////
  void CC::use_lambda ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    _use_lambda = true;
  }

/////////////////////////////////////////////////////////////////////////////////////
  bool CC::lambda_used ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return _use_lambda;
  }

/////////////////////////////////////////////////////////////////////////////////////
  char * CC::lepton ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return lepton_name;
  }

/////////////////////////////////////////////////////////////////////////////////////
  void CC::set_hadron_mass (double hadron_mass)
/////////////////////////////////////////////////////////////////////////////////////
  {
    M = hadron_mass;
    MM = M * M;
  }

/////////////////////////////////////////////////////////////////////////////////////
  double CC::hadron_mass ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return M;
  }


/////////////////////////////////////////////////////////////////////////////////////
  double CC::Enu0 ()
/////////////////////////////////////////////////////////////////////////////////////
  {
    return (mm + 2 * m * M) / 2 / M;
  }

/////////////////////////////////////////////////////////////////////////////////////
  double CC::kprimmax2 (double Ek, double Ep, double pfermi)
/////////////////////////////////////////////////////////////////////////////////////
  {
    return sqrt (kwad (Ek + Ep - sqrt (MM + pfermi * pfermi)) - mm);
  }


/////////////////////////////////////////////////////////////////////////////////////
   double CC::total_sigma(double E)
/////////////////////////////////////////////////////////////////////////////////////
   { Enu=E;
    return calg5(*this,&CC::sigma,qq(-1),qq(+1),1e-41*cm2,10); 
   }
 
 
#endif
