#ifndef C_GAUSSIAN_DISTRIB_H
#define C_GAUSSIAN_DISTRIB_H

#include <cmath>

#include "CRandomGen.h"

//Gaussian distribution of variable around centerValue
//exp(-8*[(x-centerValue)/diffuseness]**2)*sqrt(8/pi)/diffuseness

class CGaussianDistrib
{
public:
	CGaussianDistrib(const double i_centerValue,
		const double i_diffuseness,
		CRandomGen& io_randomNumber)
		:m_centerValue(i_centerValue),
		 m_diffuseness(i_diffuseness),
		 m_coeff(sqrt(8.0/pi)/m_diffuseness),
		 m_rnd(io_randomNumber),
		 m_randomArgument(0.0),
		 m_value(0.0),
		 m_randomValue(0.0)
	{
	};

	double draw() const
	{	
		while(true)
		{
			m_randomArgument = 0.30*tan( pi*(m_rnd - 0.5) );
			m_value = exp(-8.0*m_randomArgument*m_randomArgument);	
			m_randomValue = m_rnd*0.32*0.30/( 0.3*0.3 + m_randomArgument*m_randomArgument);

			if(m_randomValue <= m_value)
				return m_centerValue + m_randomArgument*m_diffuseness;
		}
	};

	double eval(const double removalE) const
	{
		const double argument( (removalE-m_centerValue)/m_diffuseness );
		return m_coeff*exp( -8.0*argument*argument );	
	};

private:
	const double m_centerValue;
	const double m_diffuseness;
	const double m_coeff;
	CRandomGen m_rnd;
	mutable double m_randomArgument;
	mutable double m_value;
	mutable double m_randomValue;

};


#endif
