#ifndef C_GAUSSIAN_SF_CA40_H
#define C_GAUSSIAN_SF_CA40_H

#include "CSpectralFunc.h"

#include "CGaussianDistrib.h"
#include "CMomDistrib.h"
#include "CRandomGen.h"

#include "GConstants.h"

//Gaussian approximation of the spectral function of 40Ca

class CGaussianSF_Ca40: public CSpectralFunc
{
public:
	CGaussianSF_Ca40(const MomDistribs i_momDistrib, 
			const IsospinOfSF i_isospin, 
			CRandomGen& io_rnd);

	const CMomDistrib* m_momDistrib;

	double evalSF(const double p, const double removE) const;

	double evalMFMomD(const double p) const;
	double evalCorrMomD(const double p) const;
	double evalTotMomD(const double p) const;

	double generateMom() const;

	double get_pBlock() const;
	double get_pMin() const;
	double get_pMax() const;
	double get_eMin() const;
	double get_eMax() const;
	double get_targetAlpha() const;


private:
	CRandomGen m_rnd;
	const IsospinOfSF m_isospin;

	const double m_targetBeta;
	const double m_targetE2;
	const double m_pBlock;
	const double m_pStart;
	const double m_pStop;
	const double m_eStart;
	const double m_eStop;

	mutable double m_targetAlpha;
	mutable double m_coeff;

	mutable double m_TEnergy[numberOfCaLevels];
	mutable double m_TWidth[numberOfCaLevels];

	const CGaussianDistrib* m_1sShell1_2;
	const CGaussianDistrib* m_1pShell3_2;
	const CGaussianDistrib* m_1pShell1_2;
	const CGaussianDistrib* m_1dShell5_2;
	const CGaussianDistrib* m_2sShell1_2;
	const CGaussianDistrib* m_1dShell3_2;

};

#endif
