#ifndef C_LORENTZ_DISTRIB_H
#define C_LORENTZ_DISTRIB_H

#include <cmath>

#include "CRandomGen.h"

//Lorentz distribution of variable:
//halfWidth/( (x-centerValue)**2 + (halfWidth)**2)

class CLorentzDistrib
{
public:
	CLorentzDistrib(
		const double i_centerValue, 
		const double i_halfWidth, 
		CRandomGen& io_randomNumber)
		:m_pi( 4*atan(1.0) ),
		 m_centerValue(i_centerValue),
		 m_halfWidth(i_halfWidth),
		 m_rnd(io_randomNumber)
	{
	};

	inline double eval(const double i_x) const
	{
		double arg( (m_centerValue - i_x)/m_halfWidth );
		return 1.0/( ( 1.0 + arg*arg )*m_halfWidth );	
	};	


	inline double generate() const
	{	
		return m_centerValue + m_halfWidth*tan( m_pi*(m_rnd - 0.5) );	
	};

	inline double generatePositiv() const
	{	
		return m_centerValue + m_halfWidth*tan( 0.5*m_pi*m_rnd );	
	};

private:
	const double m_pi;
	const double m_centerValue;
	const double m_halfWidth;
	CRandomGen m_rnd;

};


#endif
