#ifndef C_SF_OPTIONS_H
#define C_SF_OPTIONS_H

#include <iostream>
#include "CRandomGen.h"
#include "GConstants.h"
#include "FormFactors.h"

typedef double (*DFormFactorPtr)(const double);

class CSFOptions
{
public:
	CSFOptions(const TargetElement i_target,
		const IsospinOfSF i_isospin,
		const MomDistribs i_momDistrib,
		const ScatteredNeutrinoFlav i_neutrinoFlavour,
		const FormFactors i_formFactors,
		const bool i_switchToEM,
		const bool i_switchPauliBlocking,
		const double i_neutrinoE,
		const int i_numberOfRepetitions);

	TargetElement get_target() const;
	IsospinOfSF get_isospin() const;
	MomDistribs get_momDistrib() const;

	double get_leptonMass() const;
	double get_leptonMass2() const;
	std::string get_neutrinoFlav() const;
	bool get_switchAntineut() const;
	
	DFormFactorPtr get_FFGE() const;
	DFormFactorPtr get_FFGM() const;

	int get_numberOfRepetitions() const;

	bool get_switchToEM() const;
	bool get_switchPauliBlocking() const;

	double get_neutrinoE() const;

	double evalLH(const double q4til2, 
				  const double p4k4, 
				  const double p4kPrime4, 
				  const double p4q4til,
				  const double k4q4til,
				  const double kPrime4q4til,
				  const double k4kPrime4) const;


private:
	const TargetElement m_target;
	const IsospinOfSF   m_isospin;
	const MomDistribs m_momDistrib;

	mutable double m_leptMass;
	mutable double m_leptMass2;
	mutable std::string m_leptFlav;
	mutable bool m_switchAntineut;

	mutable DFormFactorPtr FFGE;
	mutable DFormFactorPtr FFGM;

	const int m_numberOfRepetitions;

	const bool m_switchToEM;
	const bool m_switchPauliBlocking;

	const double m_neutrinoE;

	const FormFactors m_formFactors;

	mutable int m_N;//number of neutrons
	mutable int m_Z;//number of protons

};


#endif
