#include "CSpectralFunc.h"

#include "CSpectralFunc_C12.h"
#include "CSpectralFunc_O16.h"
#include "CSpectralFunc_Fe56.h"
#include "CSpectralFunc_Ar40.h"
#include "CGaussianSF_O16.h"
#include "CGaussianSF_Ar40.h"
#include "CGaussianSF_Ca40.h"

CSpectralFunc* createSF(const TargetElement i_target,
					    const MomDistribs i_momDistrib,
					    const IsospinOfSF i_isospin,
					    CRandomGen& io_rnd)
{
	switch (i_target)
	{
		case targC_Ben:
			return new CSpectralFunc_C12(io_rnd);

		case targO_Ben:
			return new CSpectralFunc_O16(io_rnd);

		case targO_GSF:
			return new CGaussianSF_O16(i_momDistrib, i_isospin, io_rnd);

		case targCa_GSF:
			return new CGaussianSF_Ca40(i_momDistrib, i_isospin, io_rnd);

		case targAr_GSF:
			return new CGaussianSF_Ar40(i_momDistrib, i_isospin, io_rnd);

		case targAr_default:
			return new CSpectralFunc_Ar40(i_isospin, io_rnd);

		case targFe_Ben:
			return new CSpectralFunc_Fe56(io_rnd);
			
		default:
			return 0;

	}
}

