#include "CSpectralFunc_Fe56.h"

//Spectral function of carbon normalized to 1
//value at boundries is zero

CSpectralFunc_Fe56::CSpectralFunc_Fe56(CRandomGen& io_rnd)
			:m_eRes(80),
			m_pRes(40),
			m_eStep(5.0),
			m_pStep(20.0),
			m_eStart(0.0),
			m_pStart(0.0),
			m_eStop(400.0),
			m_pStop(800.0),
			m_pBlock(ironPBlock),
			m_normal(1.0771/ironZ),
			m_targetAlpha( 3.0/(4.0*beta_Fe56*avPMF_Fe56_Ben*avPMF_Fe56_Ben) ),
			m_SFTable(m_pRes,m_eRes)
{
	m_momDistrib = createDistrib(md_Fe56_Ben, proton, io_rnd);

	std::ifstream inputFile("pke56_tot.data");
	
	if (inputFile.fail())
	{
		std::cerr<<"Indispensable file 'pke56_tot.data' not found"<<std::endl;
		return;
	}

	double p(0), e(0), val(0.0);

	for (int iCnt(0); iCnt < m_pRes; ++iCnt)
	{
		inputFile>>p;

		for (int jCnt(0); jCnt < m_eRes; ++jCnt)
		{
			inputFile>>e>>val;
			m_SFTable(iCnt, jCnt) = val;
		}
	}

}



double CSpectralFunc_Fe56::evalSF(const double p, const double e) const
{
	//p = m_pStep*(np+1) + pR;

	if (p<m_pStart || p>m_pStop || e<m_eStart || e>m_eStop)
		return 0.0;

	const int    np( (p-0.5*m_pStep < 0.0) ? -1 : static_cast<int>((p-0.5*m_pStep)/m_pStep) );
	const int    ne( (e-m_eStep < 0.0) ? -1 : static_cast<int>((e-m_eStep)/m_eStep) );
	
	const double pR( p-m_pStep*(np+0.5) );
	const double eR( e-m_eStep*(ne+1.0) );

	//std::cout<<p<<" "<<np*m_pStep+m_pStep+pR<<" "<<e<<" "<<ne*m_eStep+m_eStep+eR<<std::endl;
	
	const double c00( SFTabVal(np,ne)     ); //left lower corner
	const double c10( SFTabVal(np+1,ne)   ); //right lower corner
	const double c01( SFTabVal(np,ne+1)   ); //left upper corner
	const double c11( SFTabVal(np+1,ne+1) ); //right upper corner

	double f( (c11 - c10 - c01 + c00)*eR*pR/(m_pStep*m_eStep) + (c10 - c00)*pR/m_pStep + (c01 - c00)*eR/m_eStep + c00 );

	return m_normal*f;

}



double CSpectralFunc_Fe56::evalMFMomD(const double p) const
{
	return m_momDistrib->MF(p);
}


double CSpectralFunc_Fe56::evalCorrMomD(const double p) const
{
	return m_momDistrib->Corr(p);
}


double CSpectralFunc_Fe56::evalTotMomD(const double p) const
{
	return m_momDistrib->Tot(p);
}


double CSpectralFunc_Fe56::generateMom() const
{
	return m_momDistrib->generate();
}



double CSpectralFunc_Fe56::get_pBlock() const
{
	return m_pBlock;
}


double CSpectralFunc_Fe56::get_pMin() const
{
	return m_pStart;
}


double CSpectralFunc_Fe56::get_pMax() const
{
	return m_pStop;
}


double CSpectralFunc_Fe56::get_eMin() const
{
	return m_eStart;
}


double CSpectralFunc_Fe56::get_eMax() const
{
	return m_eStop;
}


double CSpectralFunc_Fe56::get_targetAlpha() const
{
	return m_targetAlpha;
}
