#ifndef C_SPECTRAL_FUNC_O16_H
#define C_SPECTRAL_FUNC_O16_H

#include "CSpectralFunc.h"

#include <fstream>

#include "CMomDistrib.h"
#include "CRandomGen.h"

#include "GConstants.h"
#include "TArray2D.h"


//Spectral function normalized to 1
//value at boundries is zero

class CSpectralFunc_O16: public CSpectralFunc
{
public:
	CSpectralFunc_O16(CRandomGen& io_rnd);

	const CMomDistrib* m_momDistrib;

	double evalSF(const double p, const double removE) const;

	double evalMFMomD(const double p) const;
	double evalCorrMomD(const double p) const;
	double evalTotMomD(const double p) const;

	double generateMom() const;

	double get_pBlock() const;
	double get_pMin() const;
	double get_pMax() const;
	double get_eMin() const;
	double get_eMax() const;
	double get_targetAlpha() const;


private:
	const int m_eRes;
	const int m_pRes;

	const double m_eStep;
	const double m_pStep;

	const double m_eStart;
	const double m_pStart;

	const double m_eStop;
	const double m_pStop;

	const double m_pBlock;

	const double m_targetAlpha;
	
	const double m_normal;

	mutable TArray2D<double> m_SFTable;

	double SFTabVal(const int np, const int ne) const
	{
		if ( np>=0 && np<m_pRes && ne>=0 && ne<m_eRes )
			return m_SFTable(np,ne);
		return 0.0;
	}
};


#endif
