#ifndef G_CONSTANTS_H
#define G_CONSTANTS_H

#include <cmath>

enum TargetElement
{	
	targC_Ben,
	targO_Ben,
	targO_GSF,
	targCa_GSF,
	targAr_GSF,
	targAr_default,
	targFe_Ben
};

enum MomDistribs
{	
	md_C12_Ben,
	md_O16_Ben,
	md_O16_GCo,
	md_O16_CdA,	
	md_Ca40_CdA,
	md_Ca40_GCo,
	md_Ca48_GCo,
	md_Fe56_Ben
};

enum IsospinOfSF
{
	proton,
	neutron
};

enum ScatteredNeutrinoFlav
{	
	flav_e,
	flav_ae,
	flav_mu,
	flav_amu,
	flav_tau,
	flav_atau,
	flav_mless,
	flav_amless
};

enum FormFactors
{
	dipoleFF,
	BBBA05FF,
	BBA03FF,
	JLabFF
};


const double pi(4*atan(1.0));
//const double GFcosTheta(1.14959);//Fermi constant including the Cabibbo angle, *10^11
const double GF(1.16637);//Fermi constant *10^11
const double cosThetaC(0.97377);//Cabibbo angle, PDG 2006, p. 138
const double GFcosTheta(GF*cosThetaC);//Fermi constant including the Cabibbo angle, *10^11
const double alpha(7.297352568e+4);//fine structure constant, *10^7

const double pMass( 938.272029 );
const double nMass( 939.565360 );
const double M( 0.50*(pMass+nMass) );
const double piMass( 139.57018 );//charged pion mass

const double eMass(     0.510998918 );
const double muMass(  105.6583692 );
const double tauMass(1776.9 );

const double M2(M*M);
const double piMass2(piMass*piMass);

const double unit( 931.494043);
const double hbarc(197.326968);
const double hbarc2(hbarc*hbarc);
const double hbarc3(hbarc*hbarc*hbarc);

const double mu_p(2.792847351);
const double mu_n(-1.91304273);
//const double xi(mu_p - mu_n - 1.0);
const double gA(-1.2673);
//const double MA(1070.0);
const double MA(1030.0);

const double MV2(710000.0);
const double MA2(MA*MA);

const int carbonA(12);
const int carbonN( 6);
const int carbonZ( 6);

const int oxygenA(16);
const int oxygenN( 8);
const int oxygenZ( 8);

const int calciumA(40);
const int calciumN(20);
const int calciumZ(20);

const int argonA(40);
const int argonN(22);
const int argonZ(18);

const int ironA(40);
const int ironN(22);
const int ironZ(18);

const double oxygenE1(19.5855);//22.0725;//19.5855;//17.9518;//11.6152; from muclei masses//
const double oxygenE2(26.3294);

const double calciumE1(29.2565);//from muclei masses//
const double calciumE2(16.4631);//

const double argonE1(29.2565);//from muclei masses//
const double argonE2(16.4631);

const double avPMF_C12_Ben( 168.241 );
const double avPMF_O16_CdA( 162.121 );
const double avPMF_O16_Ben( 167.83 );
const double avPMF_O16_GCo( 174.4 );	
const double avPMF_Ca40_CdA(  177.59 );
const double avPMF_Ca40P_GCo( 189.07 );
const double avPMF_Ca40N_GCo( 187.01 );
const double avPMF_Ca48P_GCo( 180.75 );
const double avPMF_Ca48N_GCo( 196.42 );
const double avPMF_Fe56_Ben( 182.117 );

const double beta_C12( (carbonA-2.0)/(carbonA-1.0) );
const double beta_O16( (oxygenA-2.0)/(oxygenA-1.0) );
const double beta_Ar40( (argonA-2.0)/(argonA-1.0) );
const double beta_Ca40( (calciumA-2.0)/(calciumA-1.0) );
const double beta_Fe56( (ironA-2.0)/(ironA-1.0) );

const double carbonPBlock( 209.0 );
const double carbonEBlock( sqrt(M2+carbonPBlock*carbonPBlock) );

const double oxygenPBlock( 209.0 );
const double oxygenEBlock( sqrt(M2+oxygenPBlock*oxygenPBlock) );

const double calciumPBlock( 217.0 );
const double calciumEBlock( sqrt(M2+calciumPBlock*calciumPBlock) );

const double argonPBlock( 217.0 );
const double argonEBlock( sqrt(M2+argonPBlock*argonPBlock) );

const double ironPBlock( 225.0 );
const double ironEBlock( sqrt(M2+ironPBlock*ironPBlock) );

const int numberOfOxLevels(6);

enum OxygenLevels
{
	oxLev1s1_2,
	oxLev1p3_2,
	oxLev1p1_2,
	oxLev1d5_2,
	oxLev2s1_2,
	oxLev1d3_2
};

const double TOxygenNEnergy[numberOfOxLevels] = 
{
	 47.00,
	 21.80,
	 15.65,
	  4.15,
	  3.27,
	 -0.93
};

const double TOxygenNWidth[numberOfOxLevels] = 
{
	70.0,
	4.0,
	4.0,
	2.0,
	2.0,
	2.0
};

const double TOxygenPEnergy[numberOfOxLevels] = 
{
	 45.00,
	 18.44,
	 12.11,
	 0.59,
	 0.08,
	-4.65
};
const double TOxygenPWidth[numberOfOxLevels] = 
{
	70.0,
	4.0,
	4.0,
	2.0,
	2.0,
	2.0
};

const int numberOfCaLevels(6);

enum CalciumLevels
{
	caLev1s1_2,
	caLev1p3_2,
	caLev1p1_2,
	caLev1d5_2,
	caLev2s1_2,
	caLev1d3_2
};


const double TCalciumNEnergy[numberOfCaLevels] = 
{
	66.12,
	43.80,
	39.12,
	22.48,
	17.53,
	15.79
};

const double TCalciumNWidth[numberOfCaLevels] = 
{
	25.0,//"exact"
	15.0,//"exact"
	15.0,//"exact"
	6.0,//"exact"
	4.0,
	4.0
};


const double TCalciumPEnergy[numberOfCaLevels] = 
{
	
	57.38,
	36.52,
	31.62,
	14.95,
	10.67,
	8.88
};

const double TCalciumPWidth[numberOfCaLevels] = 
{
	25.0,//"exact"
	15.0,//"exact"
	15.0,//"exact"
	4.0,//"exact"
	2.0,
	2.0
};

const int numberOfArLevels(7);
const int numberOfArNLevels(7);
const int numberOfArPLevels(6);

enum ArgonLevels
{
	arLev1s1_2,
	arLev1p3_2,
	arLev1p1_2,
	arLev1d5_2,
	arLev2s1_2,
	arLev1d3_2,
	arLev1f7_2
};

const double TArgonNEnergy[numberOfArLevels] = 
{
	62.0,
	40.0,
	35.0,
	18.0,
	13.15,//Ar theory
	11.45,//Ar theory
	5.56//Ar theory
};

const double TArgonNWidth[numberOfArLevels] = 
{
	25.0,
	15.0,
	15.0,
	5.0,//
	4.0,//
	3.0,//
	3.0//
};


const double TArgonPEnergy[numberOfArLevels] = 
{
	52.0,
	32.0,
	28.0,
	11.0,
	8.0,
	6.0,
	1.0
};

const double TArgonPWidth[numberOfArLevels] = 
{
	25.0,//"exact"
	15.0,//"exact"
	15.0,//"exact"
	4.0,//"exact"
	2.0,
	2.0,
	2.0
};


#endif
