#ifndef SCATTER_ON_SF_H
#define SCATTER_ON_SF_H

#include <fstream>

#include "GConstants.h"
#include "CVec.h"
#include "CGaussianDistrib.h"
#include "CSpectralFunc.h"

void ScatterOnSF(const CSFOptions& opt, std::ofstream& result)
{
	const double nuE( opt.get_neutrinoE() );
	const double leptM2( opt.get_leptonMass2() );
	const double kPrimeMax( sqrt(nuE*nuE - leptM2) );
	const bool switch2EM( opt.get_switchToEM() );
	const bool switchPauliBlocking( opt.get_switchPauliBlocking() );

	const double omegaStart( 0.0 );
	const double omegaStop( nuE - opt.get_leptonMass() );

	const double eps(1.0e-15); //epsilon

	if (omegaStop - omegaStart<eps)
	{
		std::cout<<"Given "<<opt.get_neutrinoFlav().c_str()<<" neutrino energy: "<<nuE<<" MeV is too low for the QE interaction"<<std::endl;
		return;
	}

	CRandomGen rng;
	CSpectralFunc* sf = createSF(opt.get_target(), opt.get_momDistrib(), opt.get_isospin(), rng);
	const double pBlock( sf->get_pBlock() );
	const double eMin( sf->get_eMin() );
	const double eMax( sf->get_eMax() );
	
	const double weakConst( GFcosTheta*GFcosTheta/(8.0*pi*pi)*0.38937966e-5 );
	//barn*10^(-17)/MeV/sr or 10^(-41) cm^2/MeV/sr or 10^(-38) cm^2/GeV/sr for weak interaction
	const double emConst( alpha*alpha*0.38937966e-5 );
	//barn*10^(-9)/MeV/sr or 10^(-33) cm^2/MeV/sr or 10^(-30) cm^2/GeV/sr for weak interaction

  	const double constant( (!switch2EM)*weakConst + (switch2EM)*emConst );
	const double multip( constant/nuE*(4.0*pi*kPrimeMax) );

	const long numOfRep( opt.get_numberOfRepetitions() );

	double sum(0.0), val(0.0), check(0.0);

	//CGaussianDistrib gauss(190.0, sqrt(8.0/sf->get_targetAlpha()), rng);
	//CGaussianDistrib gauss(0.0, sqrt(8.0)*60.0, rng);

//Setting up plot-----------------------------------------------------------------------------------
	const int numberOfBins(80);
	const double minimalValue(0.0);
	const double maximalValue(1600.0);
	const double step( (maximalValue-minimalValue)/numberOfBins );

	double* TableOfSums = new double[numberOfBins];

	for (int kCnt(0); kCnt<numberOfBins; ++kCnt)
		TableOfSums[kCnt]=0.0;

//End------------------------------------------------------------------------------------------------
	
	for(int jCounter(0); jCounter<numOfRep; ++jCounter)
	{
		const CVec k(0.0,0.0,nuE);
		const CVec kPrime(kPrimeMax, rng, false);
		const double pNorm( sf->generateMom() );
		const CVec p(pNorm, rng, true);

		const CVec q(k-kPrime);
		const double eLept( sqrt(kPrime.normSq() + leptM2) );
		const double omega( nuE - eLept );

		const double eP( sqrt(p.normSq() + M2) ); //target nucleon energy
		const double ePPrime( sqrt( (p + q).normSq() + M2) );//outgoing nucleon energy

		const double omegaTil( ePPrime - eP ); 

		if ( (omegaTil < 0.0) || (omega < 0.0) )
			continue;

		if ( switchPauliBlocking*( (p+q).norm()<pBlock ) )
			continue;

		const double q4til2( omegaTil*omegaTil - q.normSq() );
		const double p4k4( nuE*eP - k*p );
		const double eBen(omega + M - ePPrime);	

		if ( (eBen > eMax) || (eMin > eBen) )
			continue;

		if ( (p4k4 < 0.0) || (q4til2 > 0.0) )
			continue;

		const double p4kPrime4(eP*eLept - p*kPrime);
		const double p4q4til(eP*omegaTil-p*q);
		const double k4q4til(nuE*omegaTil-k*q);
		const double kPrime4q4til(eLept*omegaTil-kPrime*q);
		const double k4kPrime4(nuE*eLept-k*kPrime);

		val = multip*kPrime.normSq()/( eP*ePPrime*(nuE-omega)*sf->evalTotMomD(p.norm()) )*sf->evalSF(p.norm(),eBen)*opt.evalLH(q4til2,p4k4,p4kPrime4,p4q4til,k4q4til,kPrime4q4til,k4kPrime4);
		

//Data collecting------------------------------------------------------------------------------------------------

		const double qNorm( q.norm() );

		if ( qNorm>minimalValue && qNorm<maximalValue )
		{
			TableOfSums[ (int)( (qNorm-minimalValue)/step ) ] += val;

			check +=val;
		}

		else if ( val > eps )
			std::cout<<"Increase range! "<<qNorm<<std::endl;
			
/*		const double qNorm(q.norm());
		if (omega>minVal_omega && omega<maxVal_omega && qNorm>minVal_q && qNorm<maxVal_q)
			TableOfSums((int)( (omega-minVal_omega)/step_omega),(int)( (qNorm-minVal_q)/step_q)) += val;
		else if (val>eps)
			std::cout<<"Increase range! "<<omega<<" "<<qNorm<<std::endl;
*/
/*
		const double eLMap(730.0);
		const double eM2(eMass*eMass);
		const double cosTheta( ( 0.5*(omega*omega-q.normSq())-eM2+eLMap*(eLMap-omega) )/sqrt( (eLMap*eLMap-eM2)*((eLMap-omega)*(eLMap-omega)-eM2) ) );

		//const double cosTheta( ( omega*omega-q.normSq()-leptM2+2.0*nuE*(nuE-omega) )/( 2.0*nuE*kPrime.norm() ) );
	

		if (fabs(cosTheta)<1.0)
		{
			const double theta( acos(cosTheta)*180.0/pi ); 
		
			TableOfSums[(int)( (theta-minimalValue)/step)] += val;

			check +=val;
		}

		//else if (val>eps)
		//	std::cout<<"Increase range! "<<omega<<" "<<eLMap<<" "<<cosTheta<<std::endl;
*/
/*
		const CVec versor( k/k.norm() );//nu direction 
		//std::cout<<versor.get_x()<<" "<<versor.get_y()<<" "<<versor.get_z()<<" k: "<<k.get_x()<<" "<<k.get_y()<<" "<<k.get_z()<<"\n";

		const double nuERec( (M*eLept-0.5*leptM2)/(M-eLept+versor*kPrime) );

		if (nuERec<eLept)
			continue;
		

		const double q4rec2( leptM2+2.0*nuERec*(versor*kPrime-eLept) );

		const double Q2(-q4rec2*1.0e-6);

		if (Q2<maximalValue && Q2>minimalValue)
			TableOfSums[(int)( (Q2-minimalValue)/step)] += val;

		else if (val != 0.0)
				std::cout<<"Increase range! "<<"Q2: "<<Q2<<" nuERec: "<<nuERec<<" denomin: "<<M-eLept+versor*kPrime<<" "<<val<<std::endl;
*/			
/*
		if (Q2<maximalValue && Q2>minimalValue)
		{
		//	std::cout<<nuERec<<std::endl;
				TableOfSums[(int)( (nuERec/1000.0)/step)] += 1;
		}
*/

/*
		const double Q2( (q.normSq()-omega*omega)*1.0e-6 );

		if (Q2>maximalValue||Q2<0.0)
		{
		//	if (Q2>maximalValue)
		//		std::cout<<"Increase range! "<<Q2<<" "<<val<<std::endl;
		}

		else 
			TableOfSums[(int)( (Q2-minimalValue)/step)] += val;
*/

	/*	if (eLept<maximalValue && eLept>minimalValue)
			TableOfSums[(int)( (eLept-minimalValue)/step)] += 100*val;
		else if (val>0.0 )
			std::cout<<"Increase range! "<<" omega: "<<omega<<" omegaT: "<<omegaTilde<<" eL "<<eLept<<" "<<val<<std::endl;
		*/

	/*	if(pPrime.norm()>maximalValue||pPrime.norm()<minimalValue) 
			std::cout<<"zmien zakres! "<<ePPrime<<" "<<val<<std::endl;

		else 
			TableOfSums[(int)( (pPrime.norm()-minimalValue)/step)] += 100*val;*/
//End------------------------------------------------------------------------------------------------
	
		sum += val;

	
	}

	std::cout<<nuE<<" "<<1.0/8.0*sum/opt.get_numberOfRepetitions()<<" "<<check/sum<<std::endl;

//Beg------------------------------------------------------------------------------------------------

	for (int iCnt=0; iCnt<numberOfBins; ++iCnt)
		result<<(iCnt + 0.5)*step+minimalValue<<" "<<TableOfSums[iCnt]/(step*numOfRep)<<std::endl;

/*
	for (int iCnt=0; iCnt<numberOfBins; iCnt++)
		//result<<(iCnt + 0.5)*step+minimalValue<<" "<<TableOfSums[iCnt]/opt.get_numberOfRepetitions()/( opt.get_numberOfRepetitions() )<<std::endl;
		result<<(iCnt + 0.5)*step+minimalValue<<" "<<TableOfSums[iCnt]/(step*nuE*denomin)<<std::endl;
*/	
/*	for (int iCnt=0; iCnt<numberOfBins; iCnt++)
		//result<<(iCnt + 0.5)*step+minimalValue<<" "<<TableOfSums[iCnt]/opt.get_numberOfRepetitions()/( opt.get_numberOfRepetitions() )<<std::endl;
		result<<(iCnt + 0.5)*step+minimalValue<<" "<<TableOfSums[iCnt]/(step*opt.get_numberOfRepetitions())<<std::endl;
*/	
/*	for (int iCnt=0; iCnt<numOfBins_omega; ++iCnt)
	{
		for (int jCnt=0; jCnt<numOfBins_q; ++jCnt)
			result<<(iCnt + 0.5)*step_omega+minVal_omega<<" "<<(jCnt + 0.5)*step_q+minVal_q<<" "<<TableOfSums(iCnt,jCnt)/(step_omega*step_q*nuE)<<std::endl;
	}
	std::cout<<nuE<<" "<<1.0/8*sum/(denomin*nuE)<<std::endl;
*/	
	
	delete[]TableOfSums;

//End------------------------------------------------------------------------------------------------

	sum = 0.0;
	

}

#endif
