#ifndef _nucleus_h_
#define _nucleus_h_

#include "particle.h"
#include "pdg.h"
#include "params.h"
     
using namespace std;

/*const int k_proton=1;
const int k_neutron=0;
const int k_delta=2;
const int k_pi0=3;
const int k_pim=4;
const int k_pip=5;*/

struct target
{
    int p;
    int n;
    int pr;
    int nr;

    double r;
    double Eb;
    bool local_kf;
    double kf;
	double kf_deuter;
    //bool FG;
    //bool bodek;
    //bool SF;
    //bool free;
    //bool deuter;
    //bool deuter_proton;
    int model_target;
    double V;
    
    target(params &par):p(par.nucleus_p),
                        n(par.nucleus_n),
			Eb(par.nucleus_E_b*MeV),
			kf(par.nucleus_kf*MeV),
			//local_kf(par.nucleus_local_kf),
			//bodek(par.nucleus_bodek),
			//SF(par.nucleus_SF),
			//FG(par.nucleus_FG),
			//free(par.nucleus_free),
			//deuter(par.nucleus_deuter),
			//deuter_proton(par.nucleus_deuter_proton),
			model_target(par.nucleus_target)
    {  using namespace PDG;
       if(p+n==1)
       Eb=kf=r;

if (par.nucleus_target == 2)
	local_kf=true;
else local_kf=false;

       pr=p;
       nr=n;	 
       double A = n+p;			
       double Ap = pow(A, 1.0 / 3);
       V = sqrt(mass_proton*mass_proton + kf*kf) - mass_proton + 7*MeV;
       //potencjal, wartosc do ustalenia
       double r0 = 1.1;
       r=r0*Ap;
       //parametr zadajacy promien jadra w fermi
    }
    
    void wybicie(int pdg)
    {using namespace PDG;
     switch(pdg) 
     {case pdg_proton:   pr--;break;
      case pdg_neutron:  nr--;break;
     } 
    }  
    
    double localkf(particle &pa)
    { 
     return localkf(pa.pdg,pa.r.length());
    }

    double localkf(int pdg,double r)
    { if(p+n==1) return 0;
      else
      if(p==18 && n == 22)
          return localkf_Ar(kf);
      else 
      if(p==26 && n == 30)
          return localkf_Fe(kf);
      else 
      if(p==8 && n == 8)
          return localkf_O(kf);
      else 
        return kf;
    }  
    
    double density(int pdg=0)
    { using namespace PDG;
      double Vol=r*r*r*M_PI*4/3;
      switch(pdg)
      {
      case 0: 		return  (nr+pr)/Vol;
      case pdg_proton:  return pr/Vol;
      case pdg_neutron: return nr/Vol;
      default:          return 0;
      }
    }
          
    particle shoot(vec r)
    {return shoot();   // dummy r dependence
    }
         
    particle shoot()
    { if(p+n==1)	//if only one nucleon in the target
         {int pdg= ( p ? PDG::pdg_proton : PDG::pdg_neutron);  
	  return particle(pdg,PDG::mass(pdg));
         }
	  		//if more nucleons in the target
      int pdg=(frandom()*(pr+nr)<pr ? PDG::pdg_proton : PDG::pdg_neutron);

      particle N(pdg,PDG::mass(pdg));

	//cout<<wybor<<"  ";
//0 is free target; 1 is Fermi gas; 2 is local Fermi gas; 3 is Bodek; 4 is spectral function; 5 is deuterium; 6 is only proton in deuterium (for tests only);


	/*if (SF) wybor=4;
	if (bodek) wybor=3;
	if (local_kf) wybor=2;
	if (FG) wybor=1;
	if (free) wybor=0;*/
	
	double lkf=localkf(N.pdg, N.r.length());
	//double xx=N.momentum();
	vec nic (0,0,0);

int wybor=model_target;

	switch(wybor)
	{
	case 0: N.set_momentum(nic);
	break;
	case 1: N.set_momentum(rand_from_ball(kf));
	break;
	case 2: N.set_momentum(rand_from_ball(lkf));
	break;
	case 3: N.set_momentum(bodek_rand_from_ball(kf));
	break;
	case 4: N.set_momentum(spectral_choice_oxygen(p,n));
	break;
	case 5: N.set_momentum(deuterium ()); 
	break;
	case 6: N.set_momentum(deuterium ()); 
	break;
	
//in the future it is possible to add SF for other nuclei as well
	default: N.set_momentum(rand_from_ball(kf));
	}
        N.r=rand_from_ball(r)*fermi;
	return N;

      /*if(local_kf==0)//the choice of momentum; to be done in the simpler way
        {
//	 cout<<"N="<<N<<endl;
	 N.set_momentum(bodek ? bodek_rand_from_ball(kf) : rand_from_ball(kf));
//	 cout<<"TARGET="<<N<<" MOMENTUM="<<N.momentum()/MeV<<endl;
	 return N;
	} 
      else
        { double lkf=localkf(N.pdg, N.r.length());
	  N.set_momentum(bodek ? bodek_rand_from_ball(lkf) : rand_from_ball(lkf));
	  return N;
	}*/	

    }//the end of particle shoot
    
    
        
    bool pauli_blocking (particle &pa, double ped)    // TRUE = PARTICLE IS BLOCKED
    {
      return pauli_blocking(pa.pdg, pa.momentum(), pa.r.length(), ped);
    }

    bool pauli_blocking (int pdg, double p, double r, double ped)    // TRUE = PARTICLE IS BLOCKED
    { if(p+n==1 or not nucleon(pdg))
      return false;
	if (local_kf!=0)
	  return p<localkf(pdg,r); 
      	if (model_target==5)
	 {return p<ped;}
	else
	return p<kf;

         
    }

    
private:

    double localkf_Ar (double kf)
    { generic_localkf( kf, 3.530, 0.541, 0.176,15);
    }

    double localkf_Fe (double kf)
    { generic_localkf( kf, 4.111, 0.558, 0.163, 15);
    }

    double localkf_O (double kf)
    {generic_localkf( kf, 1.544, 0.141, 1.833, 10);
    } 
    
// by JN
///////////////////////////////////////////////////////////////////////
double generic_localkf (double kf,double C, double C1, double rho0, double r0)
///////////////////////////////////////////////////////////////////////
{
  double r, pr, z=0;

  do
    {
      r = frandom_sqr () * r0;
      z=(r-C)/C1;
      pr = rho0*(1./(1+exp(z)));
	
    }
  while (pr < frandom ()*0.2);

  double result = pow(3*M_PI*M_PI*pr/2,1./3.)*197.4*MeV;
//  cout<< "localkf="<<result <<endl;
  return result;
}
     

};    


#endif
