#ifndef _vec_h_
#define _vec_h_
#include <iostream>
#include <cmath>
using namespace std;

////////////////////////////////////////////////////////////////////////////
///   vec is a 3D vector                                                 ///
////////////////////////////////////////////////////////////////////////////

class vec
{
public:
  double x, y, z;                                 ///< Cartesian coordinates 

public:
   inline double &operator[] (int i);              ///< treat vec as double[3]
   inline vec();                                   ///< vec=0 
   inline vec (double x1, double x2, double x3);   ///< vec from coordinates
   inline double length ();                        ///< vector length
   inline vec dir();                               ///< normalized vector
   inline friend double operator* (vec a, vec b);  ///< scalar product
   inline friend double angle (vec a, vec b);      ///< angle between a and b
   inline friend vec vecprod(vec a, vec b);        ///< vector product
   inline friend vec operator+ (vec a, vec b);     ///<  a + b
   inline friend vec operator- (vec a, vec b);     ///< a - b
   inline vec operator- ();                        ///< -a
   inline friend vec operator* (double a, vec b);  ///< mult by number
   inline friend vec operator* (vec b, double a);  ///< mult by number
   inline friend vec operator/ (vec b, double a);  ///< div by number
   inline friend ostream & operator<< (ostream & o, vec v); ///< print vector
   inline friend double costhetalab (vec k, vec kprim); ///< 
   inline friend double cos (vec k, vec kprim);
   inline friend double thetalab (vec k, vec kprim);
};

////////////////////////////////////////////////////////////////////////////
//             I M P L E M E N T A T I O N                                //  
////////////////////////////////////////////////////////////////////////////

double & vec::operator[] (int i)
  {
    return *(&x + i - 1);
  }
  vec::vec():x(0),y(0),z(0)
  {
  }
  vec::vec (double x1, double x2, double x3):x (x1), y (x2), z (x3)
  {
  }
  double vec::length ()
  {
    return sqrt (x * x + y * y + z * z);
  }
  vec vec::dir()
  {
     return *this/length();
  }
double operator* (vec a, vec b)
  {
    return a.x * b.x + a.y * b.y + a.z * b.z;
  }
double angle (vec a, vec b)
  {
    return acos(a*b/sqrt((a*a)*(b*b)));
  }
vec vecprod(vec a, vec b)
  {
    return vec( a.y * b.z - b.y * a.z,
                a.z * b.x - b.z * a.x,
	        a.x * b.y - b.x * a.y);
  }
vec operator+ (vec a, vec b)
  {
    return vec (a.x + b.x, a.y + b.y, a.z + b.z);
  }
vec operator- (vec a, vec b)
  {
    return vec (a.x - b.x, a.y - b.y, a.z - b.z);
  }
vec vec::operator- ()
  {
    return vec (-x, -y, -z);
  }
vec operator* (double a, vec b)
  {
    return vec (a * b.x, a * b.y, a * b.z);
  }
vec operator* (vec b, double a)
  {
    return vec (a * b.x, a * b.y, a * b.z);
  }
vec operator/ (vec b, double a)
  {
    return vec (b.x / a, b.y / a, b.z / a);
  }
ostream & operator<< (ostream & o, vec v)
  {
    return o << '(' << v.x << ',' << v.y << ',' << v.z << ')';
  }
double costhetalab (vec k, vec kprim)
  {
    return k * kprim / sqrt ((k * k) * (kprim * kprim));
  }

double cos (vec k, vec kprim)
  {
    return k * kprim / sqrt ((k * k) * (kprim * kprim));
  }

double thetalab (vec k, vec kprim)
  {
    return acos (k * kprim / sqrt ((k * k) * (kprim * kprim))); 
  }

////////////////////////////////////////////////////////////////////////////
//              I M P L E M E N T A T I O N  -  E N D                     //  
////////////////////////////////////////////////////////////////////////////
#endif
