#ifndef _vect_h_
#define _vect_h_
#include <iostream>
#include <cassert>
#include <cmath>
#include "vec.h"
#include "jednostki.h"

using namespace std;

double frandom();

/// Minkowski 4-vector
class vect
{
public:
  double t, x, y, z;
//  friend class spinor;

public:
  inline vect (double x0 = 0, double x1 = 0, double x2 = 0, double x3 = 0);
  inline vect (vec v0, double t0=0);
  inline vec operator= (vec v);
  inline vect operator+= (vect v);
  inline vect operator-= (vect v);
  inline vect operator+= (vec v);
  inline vect operator-= (vec v);
  inline operator vec ();
  inline double &operator[] (int i);
  inline friend double operator* (vect a, vect b);    ///< scalar product   
  inline friend vect operator* (double a, vect b);    ///< multiply 4-vector by a number
  inline friend vect operator* (vect b, double a);    ///< multiply 4-vector by a number
  inline friend vect operator/ (vect b, double a);    ///< divide 4-vector by a number
  inline friend vect operator+ (vect a, vect b);      ///< add 4-vectors
  inline friend vect operator- (vect a, vect b);      ///< subtract 4-vectors
  inline vect operator- ();                           ///< negate 
  inline vec speed ();                                ///< momentum / energy
  inline vec v ()    ;                                ///< momentum / energy
  inline vect & boost (vec v);                        ///< boost to a frame moving with velocity v
  inline vect & boost2 (vec v);                       ///< boost2 to a frame moving with velocity v
  inline friend ostream & operator<< (ostream & o, vect v); ///< print a 4 vector
  inline double length();                             ///< length of the space component
};

inline vec speed (vect a, vect b);            ///< CMS speed of a and b
inline vec rand_from_ball (double r);         ///< random vector from ball of radius r
inline vec rand_dir ();                       ///< random unit vector in 3D space  
inline vec bodek_rand_from_ball (double r);   ///< punkt ze sfery z ogonem Bodka
inline vec spectral_choice_oxygen (int p, int n);
inline vec deuterium ();
inline double q2 (vect k, vect kprim);
inline double q0 (vect k, vect kprim);


////////////////////////////////////////////////////////////////
///            I M P L E M E N T A C J A                     ///  
////////////////////////////////////////////////////////////////

vect::vect (double x0, double x1, double x2, double x3)
           :t (x0), x (x1), y (x2), z (x3)
  {
  }
vect::vect (vec v0, double t0):t (t0), x (v0.x), y (v0.y), z (v0.z)
  {
  }
double & vect::operator[] (int i)
  {
    return *(&t + i);
  }
double operator* (vect a, vect b)
  {
    return a.t * b.t - a.x * b.x - a.y * b.y - a.z * b.z;
  }
vect operator+ (vect a, vect b)
  { 
    return vect (a.t + b.t, a.x + b.x, a.y + b.y, a.z + b.z);
  }
vect operator- (vect a, vect b)
  {
    return vect (a.t - b.t, a.x - b.x, a.y - b.y, a.z - b.z);
  }
vect vect::operator- ()
  {
    return vect (-t, -x, -y, -z);
  }
vec vect::operator= (vec v)
  {
    x = v.x;
    y = v.y;
    z = v.z;
    return v;
  }
vect vect::operator+= (vect v)
  {
    x+=v.x;
    y+=v.y;
    z+=v.z;
    t+=v.t;
    return *this;
  }
vect vect::operator-= (vect v)
  {
    x-=v.x;
    y-=v.y;
    z-=v.z;
    t-=v.t;
    return *this;
  }
vect vect::operator+= (vec v)
  {
    x+=v.x;
    y+=v.y;
    z+=v.z;
    return *this;
  }
vect vect::operator-= (vec v)
  {
    x-=v.x;
    y-=v.y;
    z-=v.z;
    return *this;
  }
vect operator* (double a, vect b)
  {
    return vect (a * b.t, a * b.x, a * b.y, a * b.z);
  }
vect operator* (vect b, double a)
  {
    return vect (a * b.t, a * b.x, a * b.y, a * b.z);
  }
vect operator/ (vect b, double a)
  {
    return vect (b.t / a, b.x / a, b.y / a, b.z / a);
  }
ostream & operator<< (ostream & o, vect v)
  {
    return o << '(' << v.t << ',' << v.x << ',' << v.y << ',' << v.z << ')';
  }
vect::operator vec ()
  {
    return vec (x, y, z);
  }  
vec vect::speed ()
  { 
    return vec (x / t, y / t, z / t);
  }

vec vect::v ()
  { 
    return vec (x / t, y / t, z / t);
  }
vect & vect::boost (vec v)
  {//cout<<v<<endl;				
    double vv = v * v;
    if(vv==0) return *this;
    if(vv>=1) 
       cout<<v<<vv<<flush;
    assert (vv < 1);
//     double old_norm=*this * *this;
    vec p = *this;
    //double dv=sqrt(vv);
    vec zmienne = (p * v) * v / vv;
    vect stale = vect (p - zmienne);
    *this = *this - stale;
    double gamma = 1 / sqrt (1 - vv);
    *this =
      vect (t + x * v.x + y * v.y + z * v.z, x + t * v.x, y + t * v.y,
	    z + t * v.z) * gamma + stale;
//     double new_norm=*this * *this;
//     cout<<"Niedokadno�boostowania="<<old_norm-new_norm<<"/"<<old_norm<<endl;
    return *this;
  }

vect & vect::boost2(vec v)
  {				
    return boost(-v);
  }

double vect::length()
  { 
    return sqrt(x*x+y*y+z*z);
  }



///////////////////////////////////////////////////////////////////////////
///                  FUNKCJE POMOCNICZE
///////////////////////////////////////////////////////////////////////////

vec
speed (vect a, vect b)
{
  vect c = a + b;
  return c.speed ();
}

vec
rand_from_ball (double r)
{				//losujemy kierunek punkt z kuli o promieniu r
  double xx, yy, zz, d;
  do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
    }
  while ((d = xx * xx + yy * yy + zz * zz) > 1);
  // i mnoymy przez r
  return vec (r * xx, r * yy, r * zz);
}

vec spectral_choice_oxygen (int p, int n)   //there is a room for dependence on nucleus (p,n)
{ 
double pp=1000;

do{
double x=frandom();
if (x>=0 && x<0.1)
pp=1062.09*sqrt(x+304.04*x*x-833.65*x*x*x)-1249.86*log(1+13.2414*x+105.915*x*x);

if (x>=0.1 && x<0.8)
pp=61.9663+x*326.887-x*x*412.57+x*x*x*360.456;

if (x>=0.8 && x<=1)
pp=256.238/(1.0-x+0.17818) + 0.00419236/(1-x+0.0056208)/(1-x+0.0056208) + 952.842 -
1726.59*x;
}while (pp>600);
double xx, yy, zz, d;
do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
    }
  while ((d = xx * xx + yy * yy + zz * zz) > 1);
  // i mnoymy przez r
  return vec (pp * xx, pp * yy, pp * zz);
}


vec deuterium ( )
{
double x=frandom();
double pp = 113.049*sqrt(x) - 129.565*x +166.956*x*x + 3.21967/(1.01-x);
//cout<<pp<<"  ";
double xx, yy, zz, d;
do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
    }
  while ((d = xx * xx + yy * yy + zz * zz) > 1);
  // i mnoymy przez r
  return vec (pp * xx, pp * yy, pp * zz);
}

vec
rand_dir ()
{				//losowy wektor jednostkowy
  double xx, yy, zz, d;
  do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
    }
  while ((d = xx * xx + yy * yy + zz * zz) > 1 || (d<0.25));
  d=sqrt(d);
  return vec (xx/d, yy/d, zz/d);
}


vec
bodek_rand_from_ball (double r)
{//losujemy kierunek punkt z kuli jednostkowej z ogonem
  double xx, yy, zz, d;
  double x=frandom();
  double a=2/GeV;
  if(x<1-6*(r*r*a*a/M_PI/M_PI))
    return rand_from_ball(r);    
  do
    {
      xx = 2 * frandom () - 1;
      yy = 2 * frandom () - 1;
      zz = 2 * frandom () - 1;
      d = xx * xx + yy * yy + zz * zz;
    }
  while (d > 1 || d <0.25 );
  x=frandom()*6*(r*r*a*a/M_PI/M_PI);
  double R=1/(1-r/(4*GeV));
  double S=6*R*a*a/Pi/Pi*r*r*r;
  double p=4*GeV*S/(S+4*GeV*x);
//  cout<<p<<endl;
  d=sqrt(d);   
  return vec (p/d * xx, p/d * yy, p/d * zz);
}

    
double
q2 (vect k, vect kprim)
{
  vect q = k - kprim;
  return q * q;
}

double
q0 (vect k, vect kprim)
{
  return k.t - kprim.t;
}



#endif
